/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.core.command;

import java.io.File;
import java.io.RandomAccessFile;
import java.util.Date;
import java.util.Vector;
import java.util.Map.Entry;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;

import re.jpayet.mentdb.core.brain.Brain;
import re.jpayet.mentdb.core.db.Record;
import re.jpayet.mentdb.core.db.file.data.Block;
import re.jpayet.mentdb.core.db.file.data.DataFile;
import re.jpayet.mentdb.core.db.file.data.DataFilePool;
import re.jpayet.mentdb.core.db.file.data.UnusedFile;
import re.jpayet.mentdb.core.db.file.index.IndexFile;
import re.jpayet.mentdb.core.db.file.index.IndexFilePool;
import re.jpayet.mentdb.core.db.file.json.JsonFormatter;
import re.jpayet.mentdb.core.db.file.json.JsonManager;
import re.jpayet.mentdb.core.db.file.node.NodeManager;
import re.jpayet.mentdb.core.db.file.transaction.Transaction;
import re.jpayet.mentdb.core.entity.concentration.ConcentrationManager;
import re.jpayet.mentdb.core.entity.data_type.DataTypeManager;
import re.jpayet.mentdb.core.entity.language.LanguageManager;
import re.jpayet.mentdb.core.entity.link.LinkManager;
import re.jpayet.mentdb.core.entity.reader_type.ReaderTypeManager;
import re.jpayet.mentdb.core.entity.circle.CircleManager;
import re.jpayet.mentdb.core.entity.relation.RelationManager;
import re.jpayet.mentdb.core.entity.stimulation.StimulationManager;
import re.jpayet.mentdb.core.entity.symbol.SymbolManager;
import re.jpayet.mentdb.core.entity.thought.ThoughtManager;
import re.jpayet.mentdb.core.entity.word.WordManager;
import re.jpayet.mentdb.core.fx.AtomFx;
import re.jpayet.mentdb.core.fx.DateFx;
import re.jpayet.mentdb.core.fx.FileFx;
import re.jpayet.mentdb.core.fx.MathFx;
import re.jpayet.mentdb.core.fx.OperatorFx;
import re.jpayet.mentdb.core.fx.StringFx;
import re.jpayet.mentdb.core.fx.TypeFx;
import re.jpayet.mentdb.core.job.JobManager;
import re.jpayet.mentdb.core.log.Log;
import re.jpayet.mentdb.core.parameter.ParameterManager;
import re.jpayet.mentdb.core.rest.REST;
import re.jpayet.mentdb.core.script.ScriptManager;
import re.jpayet.mentdb.core.search.BasicSearchEngine;
import re.jpayet.mentdb.core.search.DeepSearchEngine;
import re.jpayet.mentdb.core.sequence.SequenceManager;
import re.jpayet.mentdb.core.session.Session;
import re.jpayet.mentdb.core.session.SessionThread;
import re.jpayet.mentdb.core.session.SessionThreadAgent;
import re.jpayet.mentdb.core.session.WebSocketThread;
import re.jpayet.mentdb.core.session.env.EnvManager;
import re.jpayet.mentdb.core.stat.Statistic;
import re.jpayet.mentdb.core.statement.Statement;
import re.jpayet.mentdb.core.user.GroupManager;
import re.jpayet.mentdb.core.user.UserManager;
import re.jpayet.mentdb.doc.MQLDocumentation;
import re.jpayet.mentdb.server.Start;
import re.jpayet.mentdb.tools.Misc;

//Command
public class Command {

	//Execute the command
	public static void execute(String user, String inputText, Vector<Vector<MQLValue>> inputVector) throws Exception {
		
		String result = "";
		boolean err = false;
		long seconds = 0;
		
		if (Session.allSessions.get(user).mqlQueryMode) {
			
			if (!Session.allSessions.get(user).isGrantedApiMql) {
				
				Session.mentdbSendMessageToMe(user, 0, "Sorry, '"+user+"' is not in 'api-mql' group.", seconds);
				return;
				
			}
			
			try {
				
				//MentDB mode
				Date d1 = new Date();
				result = Session.allConnections.get(user).execute(inputText);
				Date d2 = new Date();
				seconds = (d2.getTime()-d1.getTime())/1000;
				
			} catch (Exception e) {
				
				err = true;
				result = ""+e.getMessage();
				
			}
			
			if (err) Session.mentdbSendMessageToMe(user, 0, result, seconds);
			else {
				if (result==null) {
					Session.mentdbSendMessageToMe(user, 1, result, seconds);
				} else if (result.equals("jajidfm62mr7i8dtyfr2tyrypzea8tyyoejht1")) {
					Session.close(user);
				} else {
					Session.mentdbSendMessageToMe(user, 1, result, seconds);
				}
			}
			
		} else {
			
			if (!Session.allSessions.get(user).isGrantedApiAi) {
				
				Session.mentdbSendMessageToMe(user, 0, "Sorry, '"+user+"' is not in 'api-ai' group.", seconds);
				return;
				
			}
		
			for(int i=0;i<inputVector.size();i++) {
				
				try {

					Date d1 = new Date();
					result = executeCmd(user, Misc.vectorToStringMsg(inputVector.get(i)), inputVector.get(i));
					Date d2 = new Date();
					seconds = (d2.getTime()-d1.getTime())/1000;
					
				} catch (Exception e) {
					
					err = true;
					result = ""+e.getMessage();
					
				}
				
				if (result!=null) {
				
					if (err) Session.mentdbSendMessageToMe(user, 0, result, seconds);
					else Session.mentdbSendMessageToMe(user, 1, result, seconds);
					
				}
				
			}
			
		}
		
	}

	//Execute the command
	public static String executeCmd(String user, String inputText, Vector<MQLValue> inputVector) throws Exception {
		
		//Brain mode
		
		if (!Brain.listening) {

			throw new Exception("Sorry, the brain is stopped.");
			
		} else {
		
			//Add the stimulation
			StimulationManager.addIntoTheUserStack(System.currentTimeMillis(), user, "ai", inputVector, inputText);
			return null;
			
		}

	}

	//Execute the command
	public static synchronized String executeAllCommands(boolean manageCommitOrRollback, SessionThread session, Vector<Vector<MQLValue>> inputVectorCmds, EnvManager env) throws Exception {
		
		//Initialization
		String result = "";
		
		try {
			
			//Parse all commands
			for(int i=0;i<inputVectorCmds.size();i++) {

				if (inputVectorCmds.get(i).size()>0 && !inputVectorCmds.get(i).get(0).equals(";")) {
					
					result = Command.execute(session, inputVectorCmds.get(i), env);
					
				}

			}

			//Commit the transaction
			if (manageCommitOrRollback && !session.user.equals("")) {
				Transaction.commit(session.user);
			}
			
			return result;

		} catch (Exception e) {
			
			//Rollback the transaction
			if (manageCommitOrRollback && !session.user.equals("")) {
				Transaction.rollback(session.user);
			}

			throw new Exception(e.getMessage());

		}

	}

	//Execute the command
	@SuppressWarnings({ "unchecked" })
	public static String execute(SessionThread session, Vector<MQLValue> inputVector, EnvManager env) throws Exception {
		
		//Check if this is a comment
		if (inputVector.get(0).value.startsWith("#")) {
			
			if (inputVector.get(0).value.startsWith("#:")) System.out.println(inputVector.get(0).value);
			
			return "This is a comment ...";
		}
		
		//Add ; at the end
		if (!(inputVector.get(inputVector.size()-1).value).equals(";")) {
			inputVector.add(new MQLValue(";", 0));
		}
		
		//Return the variable value
		if (inputVector.size()==2) {
			if (inputVector.get(0).cmdBlockType==1) {
				if (env.exist(inputVector.get(0).value)) 
					return env.get(inputVector.get(0).value);
				else throw new Exception("Sorry, the variable name "+inputVector.get(0).value+" does not exist.");
			}
		}
		
		//Load variables
		for(int i=0;i<inputVector.size()-1;i++) {
			if (inputVector.get(i).cmdBlockType==1) {
				//Get a variable name
				if (env.exist(inputVector.get(i).value)) 
					inputVector.get(i).value = env.get(inputVector.get(i).value);
				else {
					throw new Exception("Sorry, the variable name "+inputVector.get(i).value+" does not exist.");
				}
			} else if (inputVector.get(i).cmdBlockType==-1) {
				inputVector.get(i).value = LinkManager.get_value(inputVector.get(i).value);
			} else if (inputVector.get(i).cmdBlockType==2) {
				
				inputVector.get(i).value = Command.executeAllCommands(false, session, Misc.splitCommand(inputVector.get(i).value), env);
				
			}
		}

		if (inputVector.get(0).value.equals("concat") && inputVector.size()>1) {
			
			//Script execution
			inputVector.remove(0);
			inputVector.remove(inputVector.size()-1);
			
			return StringFx.concat(session, inputVector, env);
			
		} else if (inputVector.get(0).value.equals("case") && inputVector.size()>1) {

			//Script execution
			inputVector.remove(0);
			inputVector.remove(inputVector.size()-1);

			return Statement.case_statement(session, inputVector, env);
			
		} else if (inputVector.get(0).value.equals("+") && inputVector.size()>1) {
			
			//Script execution
			inputVector.remove(0);
			inputVector.remove(inputVector.size()-1);
			return OperatorFx.add(session, inputVector, env);
			
		} else if (inputVector.get(0).value.equals("execute") && inputVector.size()>1) {
			
			//Script execution
			inputVector.remove(0);
			inputVector.remove(inputVector.size()-1);
			return ScriptManager.execute(session, inputVector, new EnvManager());
			
		} else if (inputVector.get(0).value.equals("include") && inputVector.size()>1) {
			
			//Script execution
			inputVector.remove(0);
			inputVector.remove(inputVector.size()-1);
			return ScriptManager.execute(session, inputVector, env);
			
		} else {
			
			switch (inputVector.size()-1) {
			case 1:
				
				switch (inputVector.get(0).value) {
				case "commit":
					
					Transaction.commit(session.user);
					
					return "Commit done.";
	
				case "rollback":
					
					Transaction.rollback(session.user);
					
					return "Rollback done.";
				
				case "exit": case "quit": case "bye":
					
					return "jajidfm62mr7i8dtyfr2tyrypzea8tyyoejht1";
	
				case "true": 
					
					return "1";

				case "false": 
					
					return "0";

				case "null": 
					
					return null;

				case "name":
					
					return Start.AI_FIRST_NAME+" "+Start.AI_LAST_NAME;
				
				case "@chat": 
					
					try {
						
						Session.allSessions.get(session.user).mqlQueryMode=false;
						
						return "CHAT mode activated (do not use directly with the editor, only in your application with the web socket driver).";
					} catch (Exception e) {
						if (!Session.allSessions.containsKey(session.user)) throw new Exception("Sorry, the user '"+session.user+"' is not connected through web-socket canal.");
						else throw new Exception("Sorry, an error appears: "+e.getMessage());
					}

				case "@sid": 
					
					try {
						return ""+Session.allSessions.get(session.user).idConnection;
					} catch (Exception e) {
						if (!Session.allSessions.containsKey(session.user)) throw new Exception("Sorry, the user '"+session.user+"' is not connected through web-socket canal.");
						else throw new Exception("Sorry, an error appears: "+e.getMessage());
					}

				case "@cmdid": 
					
					try {
						return ""+Session.allSessions.get(session.user).cmdId;
					} catch (Exception e) {
						if (!Session.allSessions.containsKey(session.user)) throw new Exception("Sorry, the user '"+session.user+"' is not connected through web-socket canal.");
						else throw new Exception("Sorry, an error appears: "+e.getMessage());
					}

				case "@sessions": 
					
					JSONArray jsonResult = new JSONArray();
					
					for (Entry<Long, WebSocketThread> e : WebSocketThread.allSessions.entrySet()) {
						
						if (!e.getValue().user.equals("")) {
							JSONObject sess = new JSONObject();
							sess.put("user", e.getValue().user);
							sess.put("nbExecution", e.getValue().cmdId);
							sess.put("id", e.getValue().idConnection);
							sess.put("life", ((Start.SESSION_TIMEOUT/1000)-(e.getValue().session.getIdleTimeout())/1000)+"/"+Start.SESSION_TIMEOUT/1000+"s");
							sess.put("used", ((Start.SESSION_TIMEOUT/1000)-(e.getValue().session.getIdleTimeout())/1000));
							sess.put("maxUsable", Start.SESSION_TIMEOUT/1000);
							jsonResult.add(sess);
						}
						
					}
					
					return JsonFormatter.format(jsonResult.toJSONString());
					
				case "who": 
					
					return session.user;
	
				case "sid": 
					
					return ""+session.idConnection;
	
				case "id": 
					
					return Misc.load("data"+File.separator+".id").replace("\r", "").replace("\n", "");
	
				case "version": 
					
					return Start.version;
	
				case "cmdid": 
					
					return ""+session.nbExecution;
	
				case "sessions": 
					
					jsonResult = new JSONArray();
	
					for (Entry<Long, SessionThreadAgent> e : SessionThreadAgent.allServerThread.entrySet()) {
	
						if (!e.getValue().serverThread.user.equals("")) {
							JSONObject sess = new JSONObject();
							sess.put("user", e.getValue().serverThread.user);
							sess.put("nbExecution", e.getValue().serverThread.nbExecution);
							sess.put("life", (System.currentTimeMillis()-e.getValue().serverThread.life)/1000+"/"+Start.SESSION_TIMEOUT/1000+"s");
							sess.put("used", (System.currentTimeMillis()-e.getValue().serverThread.life)/1000);
							sess.put("maxUsable", Start.SESSION_TIMEOUT/1000);
							sess.put("id", e.getValue().serverThread.idConnection);
							jsonResult.add(sess);
						}
	
					}
	
					return JsonFormatter.format(jsonResult.toJSONString());
	
				case "help":
					
					String result = "";
					
					
					for (Entry<String, Vector<MQLDocumentation>> e : MQLDocumentation.functions.entrySet()) {
						
						result += e.getKey().toUpperCase()+"\n";
						
						for(int i=0;i<e.getValue().size();i++) {
							
							
							//Get the current MQLDocumentation object
							MQLDocumentation currentFunction = e.getValue().get(i);
							
							result+=(currentFunction.webSocket?"@":"")+currentFunction.functionId;
							
							for(int r=0;r<currentFunction.parameters.size();r++) {
								
								result += " <"+currentFunction.parameters.get(r).name+">";
	
							}
	
							result += " : "+currentFunction.description+"\n";
							
							if (i==e.getValue().size()-1) result += "\n";
							
						}
					}
	
					return result;
	
				default:
	
					return inputVector.get(0).value;
	
				}
		
			case 2:
		
				switch (inputVector.get(0).value+" "+inputVector.get(1).value) {
				case "graph refresh":
					
					switch (session.graph_type) {
					case "S":
						return "j23i88m90m76i39t04r09y35p14a96y09e57t37"+SymbolManager.pearlNecklaceSymbols(session.user, session.graph_obj);
					case "T":
						return "j23i88m90m76i39t04r09y35p14a96y09e57t38"+WordManager.pearlNecklaceWord(session.user, session.graph_obj);
					case "R":
						return "j23i88m90m76i39t04r09y35p14a96y09e57t39"+ThoughtManager.pearlNecklaceRelations(session.user, session.graph_obj);
					default: 
							
							throw new Exception("Sorry, graph functions have never been executed.");
							
					}

				case "reader reload":
					
					ReaderTypeManager.reload(session.user);
	
					return "Reader type reloaded with successful.";

				case "reader show":
					
					return JsonFormatter.format(ReaderTypeManager.show().toJSONString());
				
				case "language first":
					
					return LanguageManager.first(session.user);
		
				case "stimulation click":
										
					if (!Brain.stimulation.containsKey(session.user)) {
						
						throw new Exception("Sorry, the user '"+session.user+"' has never initilalized his session.");
						
					}
					
					try {
						
						String a = StimulationManager.call(false, session.user, null, Brain.env, Brain.session);
						
						if (a==null) return JsonFormatter.format(StimulationManager.get(0, (JSONArray) Brain.stimulation.get(session.user)).toJSONString());
						else return JsonFormatter.format(a);
						
					} catch (Exception e) {
						Transaction.rollback(Brain.user);
						throw e;
					}
	
				case "stimulation commit":
					
					Transaction.commit("ai");
					
					return "Stimulation commited with successful.";
	
				case "json unload_all":
					
					return JsonManager.unloadall(env);
				
				case "group show":

					return NodeManager.format(GroupManager.showAllGroups());

				case "src count":

					int i1 = Integer.parseInt(FileFx.count_dir("src", ".java"));
					int i2 = Integer.parseInt(FileFx.count_lines("docs"+File.separator+"mql"+File.separator+"basic-integration.mql"));

					return (i1+i2)+"";

				case "refresh tree":

					return GroupManager.toHtml(session.user);

				case "math e":

					return MathFx.e()+"";

				case "math pi":

					return MathFx.pi()+"";

				case "date current_ms":
					
					return System.currentTimeMillis()+"";

				case "date current_ns":
					
					return System.nanoTime()+"";

				case "date curdate":
					
					return DateFx.curdate()+"";

				case "date current_date":

					return DateFx.current_date()+"";

				case "date current_time":

					return DateFx.current_time()+"";

				case "date current_timestamp":

					return DateFx.current_timestamp()+"";

				case "date curtime":

					return DateFx.curtime()+"";

				case "date curtimestamp":

					return DateFx.curtimestamp()+"";

				case "date full_systimestamp":

					return DateFx.full_systimestamp()+"";

				case "date sysdate":

					return DateFx.sysdate()+"";

				case "date systime":

					return DateFx.systime()+"";

				case "date systimestamp":

					return DateFx.systimestamp()+"";

				case "date systimestamp_min":

					return DateFx.systimestamp_min()+"";

				case "env show":
					
					return NodeManager.format(env.show().toJSONString());
					
				case "script show":
					
					return NodeManager.format(ScriptManager.show().toJSONString());
					
				case "@count sessions":
					
					return ""+WebSocketThread.allSessions.size();
					
				case "@exceeded sessions":
					
					return ""+WebSocketThread.exceededSessions;
	
				case "user show":
					
					return JsonFormatter.format(UserManager.showAll(session.user).toJSONString());
		
				case "language show":
					
					return JsonFormatter.format(LanguageManager.showAll().toJSONString());
		
				/*case "emotion show":
					
					return JsonFormatter.format(EmotionManager.showAll(session.user).toJSONString());
				*/
				case "sequence show":
					
					return JsonFormatter.format(SequenceManager.showAllSeqs());
		
				case "job show":
					
					return JsonFormatter.format(JobManager.show().toJSONString());
		
				case "parameter show":
					
					return JsonFormatter.format(ParameterManager.showAllParams(session.user).toJSONString());
		
				case "brain start":
					
					if (!GroupManager.isGrantedUser(session.user, "system", session.user)) {
						
						throw new Exception("Sorry, '"+session.user+"' is not in 'system' group."); 
						
					}
					
					if (Brain.listening) {
						
						throw new Exception("Sorry, the brain is already started.");
						
					} else {
						
						Brain.listening = true;
						Start.brain = new Brain();
						Start.brain.start();
						
						return "The brain has started successfully.";
						
					}
		
				case "brain status":
					
					if (Brain.listening) {
						return "Brain is running ...";
					} else {
						return "Brain is stopped.";
					}
		
				case "brain stop":
					
					if (!GroupManager.isGrantedUser(session.user, "system", session.user)) {
						
						throw new Exception("Sorry, '"+session.user+"' is not in 'system' group."); 
						
					}
					
					if (!Brain.listening) {
						
						throw new Exception("Sorry, the brain is already stopped.");
						
					} else {
						
						Brain.listening = false;
						//Wait the end of the thread
						while(Start.brain.isAlive()) {
							
							Thread.sleep(200);
							
						}
						
						return "The brain has stopped successfully.";
						
					}
		
				case "metric sessions":
					
					String r = "Cmd on "+Statistic.stat_tot_command.size()+" days (total: "+Statistic.countCmd()+"): "+Statistic.stat_tot_command.toJSONString()+"\n";
					r += "Session on "+Statistic.stat_tot_session.size()+" days (total: "+Statistic.countSession()+"): "+Statistic.stat_tot_session.toJSONString()+"\n";
					r += "Open sessions: "+(Transaction.logFiles.size()-1)+"/"+Start.MAX_SESSION+"\n";
					for (Entry<Long, SessionThreadAgent> e : SessionThreadAgent.allServerThread.entrySet()) {
		
						if (!e.getValue().serverThread.user.equals("")) {
							r+= "  - ";
							r+= "id: "+e.getValue().serverThread.idConnection;
							r+= ", user: "+e.getValue().serverThread.user;
							r+= ", nbExecution="+e.getValue().serverThread.nbExecution;
							r+= ", life: "+(System.currentTimeMillis()-e.getValue().serverThread.life)/1000+"/"+Start.SESSION_TIMEOUT/1000+"s";
							r+= ", used: "+(System.currentTimeMillis()-e.getValue().serverThread.life)/1000;
							r+= ", maxUsable: "+Start.SESSION_TIMEOUT/1000;
							r+= "\n";
						}
		
					}
					
					return r;
				
				case "concentration show":
					
					return NodeManager.format(ConcentrationManager.show(session.user).toJSONString());
		
				case "metric system":
					
					r = "AI: "+Start.AI_FIRST_NAME.substring(0, 1).toUpperCase()+Start.AI_FIRST_NAME.substring(1).toLowerCase()+" "+Start.AI_LAST_NAME.substring(0, 1).toUpperCase()+Start.AI_LAST_NAME.substring(1).toLowerCase()+"\n";
					r += "ID: "+Misc.load("data"+File.separator+".id").replace("\r", "").replace("\n", "")+"\n";
					r += "MentDB: v_"+Start.version+"\n";
					r += "Author: Jimmitry PAYET, Programmer Analyst\n";
					r += "Contact: contact@jpayet.re\n";
					r += "Java: v_"+Statistic.systemJavaVersion()+", "+Statistic.systemJavaVendor()+"\n";
					r += Statistic.systemName()+": v_"+Statistic.systemVersion()+", "+Statistic.systemArchitecture()+", nb_proc: "+Statistic.systemNbProcessor()+"\n";
					r += "Cpu jvm: "+Statistic.currentJvmCpuValue()+"%, Cpu sys: "+Statistic.currentSystemCpuValue()+"%, Used jvm mem: "+Statistic.currentMemJvm()+"M, Used mem: "+Statistic.getUsedPhysicalMemorySize()+"/"+Statistic.getTotalPhysicalMemorySize()+"M, Used swap mem: "+Statistic.getUsedSwapSpaceSize()+"/"+Statistic.getTotalSwapSpaceSize()+"M\n";
		
					JSONArray ar_cpu = new JSONArray();
					JSONArray ar_mem = new JSONArray();
					for(int i=0;i<Statistic.stat_cpu.size();i++) {
						ar_cpu.add(((JSONArray) Statistic.stat_cpu.get(i)).get(1));
					}
					for(int i=0;i<Statistic.stat_mem.size();i++) {
						ar_mem.add(((JSONArray) Statistic.stat_mem.get(i)).get(1));
					}
					
					r += "Cpu on 24 hours: "+ar_cpu.toJSONString()+"\n";
					r += "Mem on 24 hours: "+ar_mem.toJSONString()+"\n";
					r += "File system roots:\n";
					JSONArray array = Misc.loadArray(Statistic.fileSystemRoots());
					for(int i=0;i<array.size();i++) {
						JSONObject obj = (JSONObject) array.get(i);
						
						r += "  - ["+obj.get("absolutePath")+"] "+((100*Integer.parseInt(""+obj.get("usedSpace")))/Integer.parseInt(""+obj.get("totalSpace")))+"%";
						r += ", usedSpace: "+obj.get("usedSpace")+"M";
						r += ", freeSpace: "+obj.get("freeSpace")+"M";
						r += ", totalSpace: "+obj.get("totalSpace")+"M";
						
						r += "\n";
					};
					
					return r;
				
				case "config server":
					
					return Misc.load("conf"+File.separator+"server.conf");
				
				case "count sessions":
					
					return ""+(Transaction.logFiles.size()-1);
		
				case "exceeded sessions":
					
					return ""+Start.exceededSessions;
		
				case "metric cmd":
					
					return Statistic.stat_tot_command.toJSONString();
		
				case "metric sess":
					
					return Statistic.stat_tot_session.toJSONString();
		
				case "metric session":
					
					return Statistic.getStatSession();
		
				case "metric cpu":
					
					return Statistic.stat_cpu.toJSONString();
		
				case "metric mem":
					
					return Statistic.stat_mem.toJSONString();
		
				case "metric date":
					
					return Statistic.stat_date.toString();
		
				case "log show":
					
					return Log.show(0);
	
				case "ai firstname":
					
					return Start.AI_FIRST_NAME;
		
				case "ai lastname":
									
									return Start.AI_LAST_NAME;
						
				case "ai name":
					
					return Start.AI_FIRST_NAME+" "+Start.AI_LAST_NAME;
				
				default:
					
					switch (inputVector.get(0).value) {
					case "get_param":
						
						//Get key
						String key = inputVector.get(1).value;
		
						return ParameterManager.get_value(session.user, key);
		
					case "$":

						String str = inputVector.get(1).value;
						
						if (str.equals("[OK]")) {
							System.out.println("[OK]");
						} else {
							System.out.print(" + "+StringFx.rpad(str, " ", "58"));
						}
						
						return "1";
		
					case "stop": case "shutdown":
						
						String idCode = inputVector.get(1).value;
						
						if (!idCode.equals(Misc.load("data"+File.separator+".id").replace("\r", "").replace("\n", ""))) {
							
							throw new Exception("Sorry, the identification code is invalid."); 
							
						}
						
						if (!GroupManager.isGrantedUser(session.user, "system", session.user)) {
							
							throw new Exception("Sorry, '"+session.user+"' is not in 'system' group."); 
							
						}
						
						Start.listening = false;
						Start.serverSocket.close();
		
						return "jajidfm62mr7i8dtyfr2tyrypzea8tyyoejht0";
		
					case "eval":
						
						//Get parameters
						String mql = inputVector.get(1).value;
						
						return Statement.eval(session, mql, env);
		
					case "++":
						
						//Get parameters
						String varName = inputVector.get(1).value;
						
						return env.incr(varName, "1");
		
					case "--":
						
						//Get parameters
						varName = inputVector.get(1).value;
						
						return env.decr(varName, "1");
		
					case "not":
						
						//Get parameters
						String bool = inputVector.get(1).value;
						
						return OperatorFx.not(bool);

					default:
						
						//Script execution
				inputVector.remove(inputVector.size()-1);
				
				return StringFx.concat(session, inputVector, env);
						
					}
		
				}
		
			case 3:
				
				switch (inputVector.get(0).value+" "+inputVector.get(1).value+" "+inputVector.get(2).value) {
				case "fs cache size": 
					
					return Record.cache.size()+"";

				case "json show key": 
					
					return JsonManager.loaded(env);

				case "type data show":
					
					return JsonFormatter.format(DataTypeManager.show().toJSONString());
				
				case "type data reload":
					
					DataTypeManager.reload(session.user);
	
					return "Data type reloaded with successful.";

				case "@reset exceeded sessions":
					
					WebSocketThread.exceededSessions = 0;
					
					return "Exceeded sessions has been reset successfully.";
	
				case "job scheduler start":
					
					JobManager.scheduler_start();
					
					return "Job scheduler started with successful.";
	
				case "job scheduler stop":
					
					JobManager.scheduler_stop();
					
					return "Job scheduler stopped with successful.";
	
				case "script unload accelerator":
					
					ScriptManager.unload_accelerators();
					
					return "Script accelerator unloaded with successful.";
	
				case "job scheduler restart":
					
					JobManager.scheduler_restart();
					
					return "Job scheduler restarted with successful.";
	
				case "job scheduler status":
					
					return JobManager.scheduler_status();
				
				case "metric java vendor":
					
					return Statistic.systemJavaVendor();
		
				case "metric java version":
					
					return Statistic.systemJavaVersion();
		
				case "metric system name":
					
					return Statistic.systemName()+"";
		
				case "metric system architecture":
					
					return Statistic.systemArchitecture()+"";
		
				case "metric system version":
					
					return Statistic.systemVersion()+"";
		
				case "metric count cmd":
					
					return Statistic.countCmd()+"";
		
				case "metric count session":
					
					return Statistic.countSession()+"";
		
				case "reset exceeded sessions":
					
					Start.exceededSessions = 0;
		
					return "Exceeded sessions has been reset successfully.";
		
				case "config max session":
					
					return ""+Start.MAX_SESSION;
		
				case "config server port":
					
					return ""+Start.SERVER_PORT;
		
				case "config remote access":
					
					return ""+Start.REMOTE_ACCESS;
		
				case "config session timeout":
					
					return ""+Start.SESSION_TIMEOUT;
		
				case "config first language":
					
					return Start.FIRST_LANGUAGE;
		
				case "metric total mem":
					
					return Statistic.getTotalPhysicalMemorySize()+"";
		
				case "log current id":
					
					return Log.currentLogFileId+"";
				
				default: 
		
					switch (inputVector.get(0).value+" "+inputVector.get(1).value) {
					case "relation translate":
						
						String relationId = inputVector.get(2).value;
						
						return RelationManager.translate(session.user, relationId, LanguageManager.first(session.user));
		
					case "node insert": case "node create": case "node add":
						
						//Get key
						String key = inputVector.get(2).value;
		
						NodeManager.insertNode(session.user, key);
		
						return "Node "+inputVector.get(2).value+" created with successful.";
		
					case "node exist":
						
						//Get key
						key = inputVector.get(2).value;
		
						if (NodeManager.existNode(session.user, key)) return "1";
						else return "0";
		
					case "word show":
						
						String word = inputVector.get(2).value;
						
						return JsonFormatter.format(WordManager.showTabLinks(session.user, word).toJSONString());
						
					case "circle delete":
						
						relationId = inputVector.get(2).value;
						
						CircleManager.deleteCircle(session.user, relationId);
						
						return "The relation is out of the circle.";
		
					case "circle show":
						
						relationId = inputVector.get(2).value;
						
						return JsonFormatter.format(CircleManager.showCircle(session.user, relationId).toJSONString());
		
					case "circle ids":
						
						relationId = inputVector.get(2).value;
						
						return JsonFormatter.format(CircleManager.getIds(relationId).toJSONString());
		
					case "circle exist":
						
						relationId = inputVector.get(2).value;
						
						return CircleManager.existCircle(session.user, relationId);
		
					case "file load":
						
						String filePath = inputVector.get(2).value;
						
						return Misc.load(filePath);
		
					case "symbol stimulate":
						
						String tabLinkId = inputVector.get(2).value;
						
						SymbolManager.stimulate(session.user, tabLinkId);
						
						return "Symbol tab link stimulated with successful.";
		
					case "stimulation initialize":

						String str = inputVector.get(2).value;
						
						try {
							return JsonFormatter.format(StimulationManager.call(true, session.user, str, Brain.env, Brain.session));
						} catch (Exception e) {
							Transaction.rollback(Brain.user);
							throw e;
						}
		
					case "stimulation show":
						
						String fromUser = inputVector.get(2).value;
						
						if (!Brain.stimulation.containsKey(fromUser)) {
							
							throw new Exception("Sorry, the user '"+fromUser+"' has never initilalized his session.");
							
						}
						
						return NodeManager.format(StimulationManager.show((JSONArray) Brain.stimulation.get(fromUser)));
			
					case "stimulation size":
						
						fromUser = inputVector.get(2).value;
						
						if (!Brain.stimulation.containsKey(fromUser)) {
							
							throw new Exception("Sorry, the user '"+fromUser+"' has never initilalized his session.");
							
						}
						
						return StimulationManager.size((JSONArray) Brain.stimulation.get(fromUser));
			
					case "stimulation clear":
						
						fromUser = inputVector.get(2).value;
						
						if (!Brain.stimulation.containsKey(fromUser)) {
							
							throw new Exception("Sorry, the user '"+fromUser+"' has never initilalized his session.");
							
						}
						
						Brain.stimulation.put(fromUser, new JSONArray());
						
						return "Stimulation cleared with successful.";
			
					case "stimulation click":
						
						fromUser = inputVector.get(2).value;
						
						if (!Brain.stimulation.containsKey(fromUser)) {
							
							throw new Exception("Sorry, the user '"+fromUser+"' has never initilalized his session.");
							
						}
						
						try {
							
							String a = StimulationManager.call(false, fromUser, null, Brain.env, Brain.session);
							
							if (a==null) return JsonFormatter.format(StimulationManager.get(0, (JSONArray) Brain.stimulation.get(fromUser)).toJSONString());
							else return JsonFormatter.format(a);
							
						} catch (Exception e) {
							Transaction.rollback(Brain.user);
							throw e;
						}
		
					case "relation condition":
						
						relationId = inputVector.get(2).value;
						
						return RelationManager.checkConditionFromAnotherRelation(relationId);
		
					case "json doc":
						
						key = inputVector.get(2).value;
						
						return JsonManager.doc(env, key);
		
					case "json unload":
						
						key = inputVector.get(2).value;
						
						return JsonManager.unload(env, key);
		
					case "graph symbol":
						
						String wordName = inputVector.get(2).value;

						session.graph_type = "S";
						session.graph_obj = wordName;

						return "j23i88m90m76i39t04r09y35p14a96y09e57t37"+SymbolManager.pearlNecklaceSymbols(session.user, wordName);

					case "graph thought":
						
						wordName = inputVector.get(2).value;

						session.graph_type = "T";
						session.graph_obj = wordName;

						return "j23i88m90m76i39t04r09y35p14a96y09e57t38"+WordManager.pearlNecklaceWord(session.user, wordName);

					case "graph relation":
						
						String thoughtIdRelationId = inputVector.get(2).value;

						session.graph_type = "R";
						session.graph_obj = thoughtIdRelationId;

						return "j23i88m90m76i39t04r09y35p14a96y09e57t39"+ThoughtManager.pearlNecklaceRelations(session.user, thoughtIdRelationId);

					case "user token":
						
						String login = inputVector.get(2).value;
						
						return UserManager.get_token_from_user(login);
		
					case "in editor":
						
						str = inputVector.get(2).value;
						
						return "j23i88m90m76i39t04r09y35p14a96y09e57t36"+str;
		
					case "group add": case "group insert": case "group create":
						
						String groupName = inputVector.get(2).value;
						
						GroupManager.add(session.user, groupName);
						
						return "Group added with successful.";
		
					case "group remove": 
						
						groupName = inputVector.get(2).value;
						
						GroupManager.delete(session.user, groupName);
						
						return "Group removed with successful.";
		
					case "group exist": 
						
						groupName = inputVector.get(2).value;
						
						if (GroupManager.exist(groupName)) return "1";
						else return "0";
		
					case "group get":
						
						groupName = inputVector.get(2).value;
						
						return NodeManager.format(GroupManager.get(session.user, groupName));
		
					case "relation network":
						
						relationId = inputVector.get(2).value;
						
						return NodeManager.format(RelationManager.showNetwork(session.user, relationId).toJSONString());
					
					case "string ascii":

						String chr = inputVector.get(2).value;

						return StringFx.ascii( chr)+"";

					case "string bin":

						String num = inputVector.get(2).value;

						return StringFx.bin( num)+"";

					case "string bit_length":

						str = inputVector.get(2).value;

						return StringFx.bit_length( str)+"";

					case "string char":

						num = inputVector.get(2).value;

						return StringFx.char_str( num)+"";

					case "string char_length":

						str = inputVector.get(2).value;

						return StringFx.char_length( str)+"";

					case "string char_to_int":

						chr = inputVector.get(2).value;

						return StringFx.char_to_int( chr)+"";

					case "string encode_b64":

						String string = inputVector.get(2).value;

						return StringFx.encode_b64( string)+"";

					case "string decode_b64":

						string = inputVector.get(2).value;

						return StringFx.decode_b64( string)+"";

					case "string first_letter_upper":

						str = inputVector.get(2).value;

						return StringFx.first_letter_upper( str)+"";

					case "string first_letter":

						str = inputVector.get(2).value;

						return StringFx.first_letter(str)+"";

					case "string generate_random_str":

						String size = inputVector.get(2).value;

						return StringFx.generate_random_str( size)+"";

					case "string hex":

						num = inputVector.get(2).value;

						return StringFx.hex( num)+"";

					case "string hex_to_int":

						String hex = inputVector.get(2).value;

						return StringFx.hex_to_int( hex)+"";

					case "string hex_to_str":

						hex = inputVector.get(2).value;

						return StringFx.hex_to_str( hex)+"";

					case "string int_to_char":

						num = inputVector.get(2).value;

						return StringFx.int_to_char( num)+"";

					case "string int_to_hex":

						num = inputVector.get(2).value;

						return StringFx.int_to_hex( num)+"";

					case "string int_to_oct":

						num = inputVector.get(2).value;

						return StringFx.int_to_oct( num)+"";

					case "string is_letter":

						String value = inputVector.get(2).value;

						return StringFx.is_letter( value)+"";

					case "string is_alpha_num_uds":

						value = inputVector.get(2).value;

						return StringFx.is_alpha_num_uds( value)+"";

					case "string is_alpha_num":

						value = inputVector.get(2).value;

						return StringFx.is_alpha_num( value)+"";

					case "string is_number_char":

						value = inputVector.get(2).value;

						return StringFx.is_number_char( value)+"";

					case "string itrim":

						str = inputVector.get(2).value;

						return StringFx.itrim( str)+"";

					case "string lcase":

						str = inputVector.get(2).value;

						return StringFx.lcase( str)+"";

					case "string length":

						str = inputVector.get(2).value;

						return StringFx.length( str)+"";

					case "string lower":

						str = inputVector.get(2).value;

						return StringFx.lower( str)+"";

					case "string lrtrim":

						str = inputVector.get(2).value;

						return StringFx.lrtrim( str)+"";

					case "string lrtrim0":

						str = inputVector.get(2).value;

						return StringFx.lrtrim0( str)+"";

					case "string ltrim":

						str = inputVector.get(2).value;

						return StringFx.ltrim( str)+"";

					case "string oct":

						num = inputVector.get(2).value;

						return StringFx.oct( num)+"";

					case "string oct_to_int":

						String oct = inputVector.get(2).value;

						return StringFx.oct_to_int( oct)+"";

					case "string reverse":

						str = inputVector.get(2).value;

						return StringFx.reverse( str)+"";

					case "string rtrim":

						str = inputVector.get(2).value;

						return StringFx.rtrim( str)+"";

					case "string space":

						String count = inputVector.get(2).value;

						return StringFx.space( count)+"";

					case "string str_to_hex":

						str = inputVector.get(2).value;

						return StringFx.str_to_hex( str)+"";

					case "string to_string":

						str = inputVector.get(2).value;

						return StringFx.to_string( str)+"";

					case "string empty_if_null":

						str = inputVector.get(2).value;

						return StringFx.empty_if_null( str)+"";

					case "string null_if_empty":

						str = inputVector.get(2).value;

						return StringFx.null_if_empty( str)+"";

					case "string trim":

						str = inputVector.get(2).value;

						return StringFx.trim( str)+"";

					case "string txt":

						String data = inputVector.get(2).value;

						return StringFx.txt( data)+"";

					case "string txt2":

						data = inputVector.get(2).value;

						return StringFx.txt2( data)+"";

					case "string ucase":

						str = inputVector.get(2).value;

						return StringFx.ucase( str)+"";

					case "string unhex":

						hex = inputVector.get(2).value;

						return StringFx.unhex( hex)+"";

					case "string upper":

						str = inputVector.get(2).value;

						return StringFx.upper( str)+"";

					case "string zero":

						str = inputVector.get(2).value;

						return StringFx.zero( str)+"";

					case "type is_date":

						String date = inputVector.get(2).value;

						return TypeFx.is_date( date)+"";

					case "type is_email":

						String emailAddress = inputVector.get(2).value;

						return TypeFx.is_email( emailAddress)+"";

					case "type is_hour":

						String stringHour = inputVector.get(2).value;

						return TypeFx.is_hour( stringHour)+"";

					case "type is_time":

						stringHour = inputVector.get(2).value;

						return TypeFx.is_time( stringHour)+"";

					case "type is_hour_without_sec":

						stringHour = inputVector.get(2).value;

						return TypeFx.is_hour_without_sec( stringHour)+"";

					case "type is_number":

						value = inputVector.get(2).value;

						return TypeFx.is_number( value)+"";

					case "type is_byte":

						value = inputVector.get(2).value;

						return TypeFx.is_byte( value)+"";

					case "type is_small_int":

						value = inputVector.get(2).value;

						return TypeFx.is_small_int( value)+"";

					case "type is_medium_int":

						value = inputVector.get(2).value;

						return TypeFx.is_medium_int( value)+"";

					case "type is_int":

						value = inputVector.get(2).value;

						return TypeFx.is_int( value)+"";

					case "type is_big_int":

						value = inputVector.get(2).value;

						return TypeFx.is_big_int( value)+"";

					case "type is_float":

						value = inputVector.get(2).value;

						return TypeFx.is_float( value)+"";

					case "type is_double":

						value = inputVector.get(2).value;

						return TypeFx.is_double( value)+"";

					case "type is_timestamp":

						value = inputVector.get(2).value;

						return TypeFx.is_timestamp( value)+"";

					case "type is_valid_date":

						String dateToValidate = inputVector.get(2).value;

						return TypeFx.is_valid_date( dateToValidate)+"";

					case "type is_valid_time":

						String timeToValidate = inputVector.get(2).value;

						return TypeFx.is_valid_time( timeToValidate)+"";

					case "type is_valid_timestamp":

						String timestampToValidate = inputVector.get(2).value;

						return TypeFx.is_valid_timestamp( timestampToValidate)+"";

					case "math abs":

						String number = inputVector.get(2).value;

						return MathFx.abs( number)+"";

					case "math avg":

						String array = inputVector.get(2).value;

						return MathFx.avg(array);

					case "math acos":

						number = inputVector.get(2).value;

						return MathFx.acos( number)+"";

					case "math asin":

						number = inputVector.get(2).value;

						return MathFx.asin( number)+"";

					case "math atan":

						number = inputVector.get(2).value;

						return MathFx.atan( number)+"";

					case "math cbrt":

						number = inputVector.get(2).value;

						return MathFx.cbrt( number)+"";

					case "math ceil":

						number = inputVector.get(2).value;

						return MathFx.ceil( number)+"";

					case "math ceiling":

						number = inputVector.get(2).value;

						return MathFx.ceiling( number)+"";

					case "math cos":

						number = inputVector.get(2).value;

						return MathFx.cos( number)+"";

					case "math cosh":

						number = inputVector.get(2).value;

						return MathFx.cosh( number)+"";

					case "math cot":

						number = inputVector.get(2).value;

						return MathFx.cot( number)+"";

					case "math deg_to_rad":

						number = inputVector.get(2).value;

						return MathFx.deg_to_rad( number)+"";

					case "math exp":

						number = inputVector.get(2).value;

						return MathFx.exp( number)+"";

					case "math expm1":

						number = inputVector.get(2).value;

						return MathFx.expm1( number)+"";

					case "math floor":

						number = inputVector.get(2).value;

						return MathFx.floor( number)+"";

					case "math log":

						number = inputVector.get(2).value;

						return MathFx.log( number)+"";

					case "math log10":

						number = inputVector.get(2).value;

						return MathFx.log10( number)+"";

					case "math log1p":

						number = inputVector.get(2).value;

						return MathFx.log1p( number)+"";

					case "math rad_to_deg":

						number = inputVector.get(2).value;

						return MathFx.rad_to_deg( number)+"";

					case "math random":

						number = inputVector.get(2).value;

						return MathFx.random( number)+"";

					case "math rint":

						number = inputVector.get(2).value;

						return MathFx.rint( number)+"";

					case "math sign":

						number = inputVector.get(2).value;

						return MathFx.sign( number)+"";

					case "math signum":

						number = inputVector.get(2).value;

						return MathFx.signum( number)+"";

					case "math sin":

						number = inputVector.get(2).value;

						return MathFx.sin( number)+"";

					case "math sinh":

						number = inputVector.get(2).value;

						return MathFx.sinh( number)+"";

					case "math sqrt":

						number = inputVector.get(2).value;

						return MathFx.sqrt( number)+"";

					case "math tan":

						number = inputVector.get(2).value;

						return MathFx.tan( number)+"";

					case "math tanh":

						number = inputVector.get(2).value;

						return MathFx.tanh( number)+"";

					case "math ulp":

						number = inputVector.get(2).value;

						return MathFx.ulp( number)+"";

					case "date ts_to_long":

						String timestamp = inputVector.get(2).value;

						return DateFx.ts_to_long( timestamp)+"";

					case "date dt_to_long":

						date = inputVector.get(2).value;

						return DateFx.dt_to_long( date)+"";

					case "date day_of_month":

						date = inputVector.get(2).value;

						return DateFx.day_of_month( date)+"";

					case "date day_of_week":

						date = inputVector.get(2).value;

						return DateFx.day_of_week( date)+"";

					case "date day_of_year":

						date = inputVector.get(2).value;

						return DateFx.day_of_year( date)+"";

					case "date dayname":

						date = inputVector.get(2).value;

						return DateFx.dayname( date)+"";

					case "date hour":

						String time = inputVector.get(2).value;

						return DateFx.hour( time)+"";

					case "date is_valid_date":

						dateToValidate = inputVector.get(2).value;

						return DateFx.is_valid_date( dateToValidate)+"";

					case "date is_valid_time":

						timeToValidate = inputVector.get(2).value;

						return DateFx.is_valid_time( timeToValidate)+"";

					case "date is_valid_timestamp":

						timestampToValidate = inputVector.get(2).value;

						return DateFx.is_valid_timestamp( timestampToValidate)+"";

					case "date minute":

						time = inputVector.get(2).value;

						return DateFx.minute( time)+"";

					case "date month":

						date = inputVector.get(2).value;

						return DateFx.month( date)+"";

					case "date monthname":

						date = inputVector.get(2).value;

						return DateFx.monthname( date)+"";

					case "date seconde":

						time = inputVector.get(2).value;

						return DateFx.seconde( time)+"";

					case "date time":

						timestamp = inputVector.get(2).value;

						return DateFx.time( timestamp)+"";

					case "date year":

						date = inputVector.get(2).value;

						return DateFx.year( date)+"";

					case "is empty":
						
						//Get parameters
						str = inputVector.get(2).value;
						
						return OperatorFx.is_empty(str);
		
					case "is null":
						
						//Get parameters
						str = inputVector.get(2).value;
						
						return OperatorFx.is_null(str);
		
					case "concentration depth":
						
						key = inputVector.get(2).value;
		
						return ""+ConcentrationManager.getConcentrationDepth(key);
		
					case "log show":
						
						String nbLine = inputVector.get(2).value;
						
						//Check if the number by page is valid
						try {
	
							Integer.parseInt(nbLine);
	
						} catch (Exception e) {
	
							throw new Exception("Sorry, the number of line is not valid.");
	
						}
		
						return Log.show(Integer.parseInt(nbLine));
		
					case "transaction logs":
						
						//Initialization
						int limit = 0;
		
						//Check if the limit is valid
						try {
		
							limit = Integer.parseInt(inputVector.get(2).value);
		
							if (limit<0) {
		
								throw new Exception("err");
		
							}
		
						} catch (Exception e) {
		
							throw new Exception("Sorry, the limit is not valid.");
		
						}
		
						return JsonFormatter.format(Transaction.logList(session.user, limit).toJSONString());
		
					case "relation stimulate":
						
						//Get keys
						relationId = inputVector.get(2).value;
						
						RelationManager.stimulate(session.user, relationId, true);
		
						return "Relation "+relationId+" stimulated with successful.";
		
					case "thought stimulate":
						
						//Get keys
						String thoughtId = inputVector.get(2).value;
						
						ThoughtManager.stimulate(session.user, thoughtId);
		
						return "Thought "+thoughtId+" stimulated with successful.";
		
					case "thought show":
						
						//Get key
						word = inputVector.get(2).value;
		
						return JsonFormatter.format(ThoughtManager.list(session.user, word).toJSONString());
		
					case "relation delete":
						
						//Get relation id
						relationId = inputVector.get(2).value;
						
						RelationManager.delete(session.user, relationId);
						
						return "Relation '"+relationId+"' deleted with successful.";
		
					case "thought delete":
						
						//Get keys
						thoughtId = inputVector.get(2).value;
						
						ThoughtManager.delete(session.user, thoughtId);
		
						return "Thought "+thoughtId+" deleted with successful.";
		
					case "sequence remove":
						
						//Get key
						key = inputVector.get(2).value;
					
						SequenceManager.remove(session.user, key);
						
						return "Sequence removed with successful.";
					
					case "job remove":
						
						//Get key
						key = inputVector.get(2).value;
					
						JobManager.delete(session.user, key);
						
						return "Job removed with successful.";
					
					case "parameter remove":
						
						//Get key
						key = inputVector.get(2).value;
					
						ParameterManager.remove(session.user, key);
						
						return "Parameter removed with successful.";
					
					case "sequence increment":
						
						//Get key
						key = inputVector.get(2).value;
					
						return SequenceManager.incr(session.user, key);
					
					case "sequence exist":
						
						//Get key
						key = inputVector.get(2).value;
					
						if (SequenceManager.exist(key)) return "1";
						else return "0";
					
					case "job exist":
						
						//Get key
						key = inputVector.get(2).value;
					
						if (JobManager.exist(key)) return "1";
						else return "0";
					
					case "log trace":

						//Get text
						String text = inputVector.get(2).value;
						
						Log.write(text);
						
						return "1";
					
					case "parameter exist":
						
						//Get key
						key = inputVector.get(2).value;
					
						if (ParameterManager.exist(key)) return "1";
						else return "0";
					
					case "parameter look":
						
						//Get key
						key = inputVector.get(2).value;
					
						ParameterManager.look(session.user, key);
						
						return "Parameter looked with successful.";
					
					case "parameter unlook":
						
						//Get key
						key = inputVector.get(2).value;
					
						ParameterManager.unlook(session.user, key);
						
						return "Parameter unlooked with successful.";
					
					case "user exist":
						
						//Get key
						login = inputVector.get(2).value;
					
						if (UserManager.exist(login)) return "1";
						else return "0";
					
					case "user disable":
						
						//Get key
						login = inputVector.get(2).value;
						
						UserManager.disable(session.user, login);
					
						return "1";
					
					case "language stimulate":
						
						//Get key
						String language = inputVector.get(2).value;
						
						LanguageManager.stimulate(session.user, language);
					
						return "Language stimulated with successful.";
					
					case "user stimulate":
						
						//Get key
						login = inputVector.get(2).value;
						
						UserManager.stimulate(session.user, login);
					
						return "User stimulated with successful.";
					
					case "language insert": case "language create": case "language add": 
						
						if (!GroupManager.isGrantedUser(session.user, "system", session.user)) {
							
							throw new Exception("Sorry, '"+session.user+"' is not in 'system' group."); 
							
						}
						
						//Get parameters
						String lang = inputVector.get(2).value;
						
						LanguageManager.create(session.user, lang);
		
						return "Language "+lang+" created with successful.";
		
					case "word delete":
						
						//Get key
						word = inputVector.get(2).value;
		
						WordManager.delete(session.user, word);
		
						return "Word W["+inputVector.get(2).value+"] deleted with successful.";
		
					case "script delete":
						
						String id = inputVector.get(2).value;
						
						ScriptManager.delete(session.user, id);
						
						return "Script deleted with successful.";
					
					case "script exist":
						
						id = inputVector.get(2).value;
						
						if (ScriptManager.exist(id)) return "1";
						else return "0";
					
					case "script get":
						
						id = inputVector.get(2).value;
						
						return JsonFormatter.format(ScriptManager.get(id).toJSONString());
					
					case "node show":
						
						//Get key
						key = inputVector.get(2).value;
						
						JSONObject rec = Record.getNode(key);
		
						//SHOW RECORD
						if (rec==null) throw new Exception("Sorry, the node '"+key+"' does not exist.");
						else return JsonFormatter.format(rec.toJSONString());
		
					default:
		
						switch (inputVector.get(0).value) {
						case "->":
							
							String var = inputVector.get(1).value;
							String val = inputVector.get(2).value;
							
							env.set(var, val);
							
							return val;
						
						case ">":
							
							//Get parameters
							String number1 = inputVector.get(1).value;
							String number2 = inputVector.get(2).value;
							
							return OperatorFx.greater(number1, number2);
			
						case ">=":
							
							//Get parameters
							number1 = inputVector.get(1).value;
							number2 = inputVector.get(2).value;
							
							return OperatorFx.greater_or_equal(number1, number2);
			
						case "<":
							
							//Get parameters
							number1 = inputVector.get(1).value;
							number2 = inputVector.get(2).value;
							
							return OperatorFx.less(number1, number2);
			
						case "<=":
							
							//Get parameters
							number1 = inputVector.get(1).value;
							number2 = inputVector.get(2).value;
							
							return OperatorFx.less_or_equal(number1, number2);
			
						case "==":
			
							//Get parameters
							number1 = inputVector.get(1).value;
							number2 = inputVector.get(2).value;
							
							return OperatorFx.num_equal(number1, number2);
			
						case "!=":
							
							//Get parameters
							number1 = inputVector.get(1).value;
							number2 = inputVector.get(2).value;
							
							return OperatorFx.not_num_equal(number1, number2);
			
						case "equal":
							
							//Get parameters
							String str1 = inputVector.get(1).value;
							String str2 = inputVector.get(2).value;
							
							return OperatorFx.equal(str1, str2);
			
						case "-":
							
							//Get parameters
							number1 = inputVector.get(1).value;
							number2 = inputVector.get(2).value;
							
							return OperatorFx.sub(number1, number2);
			
						case "*":
							
							//Get parameters
							number1 = inputVector.get(1).value;
							number2 = inputVector.get(2).value;
							
							return OperatorFx.mul(number1, number2);
			
						case "/":
							
							//Get parameters
							number1 = inputVector.get(1).value;
							number2 = inputVector.get(2).value;
							
							return OperatorFx.div(number1, number2);
			
						case "and":
							
							//Get parameters
							String boolean1 = inputVector.get(1).value;
							String boolean2 = inputVector.get(2).value;
							
							return OperatorFx.and(session, boolean1, boolean2, env);
			
						case "or":
							
							//Get parameters
							boolean1 = inputVector.get(1).value;
							boolean2 = inputVector.get(2).value;
							
							return OperatorFx.or(session, boolean1, boolean2, env);
			
						case "xor":
							
							//Get parameters
							boolean1 = inputVector.get(1).value;
							boolean2 = inputVector.get(2).value;
							
							return OperatorFx.xor(boolean1, boolean2);
			
						case "if":
							
							//Get parameters
							String condition = inputVector.get(1).value;
							String trueAction = inputVector.get(2).value;
							
							return Statement.if_mql(session, condition, trueAction, env);
			
						case "while":
							
							//Get parameters
							condition = inputVector.get(1).value;
							String action = inputVector.get(2).value;
			
							return Statement.while_mql(session, condition, action, env);
			
						case "repeat":
							
							//Get parameters
							condition = inputVector.get(1).value;
							action = inputVector.get(2).value;
			
							return Statement.repeat_mql(session, condition, action, env);
			
						case "exception":
							
							//Get parameters
							id = inputVector.get(1).value;
							String message = inputVector.get(2).value;
			
							return Statement.exception_mql(session, id, message, env);
			
						case "login":
							
							String adminUser = inputVector.get(1).value.toLowerCase();
							String adminPassword = inputVector.get(2).value;
		
							if (adminUser.equals("mentdb")) {
		
								return "jajidfm62mr7i8dtyfr2tyrypzea8tyyoejht5";
		
							} else if (Transaction.logFiles.containsKey(adminUser)) {
		
								return "jajidfm62mr7i8dtyfr2tyrypzea8tyyoejht4";
		
							} else if (!adminPassword.equals("") && UserManager.checkPassword(adminUser, adminPassword)) {

								session.user = adminUser;
								if (adminUser.equals("ai")) {
									Brain.ai_connection_id = session.idConnection;
									Brain.env = session.env;
									Brain.session = session;
								}
								
								//Create the data/transaction/user folder if does not exist
								if (!(new File("data"+File.separator+"transaction"+File.separator+session.user).exists())) {
		
									(new File("data"+File.separator+"transaction"+File.separator+session.user)).mkdir();
		
								}
		
								//Open the log file
								Transaction.logFiles.put(session.user, new RandomAccessFile("data"+File.separator+"transaction"+File.separator+session.user+File.separator+"rollback.log", "rw"));
		
								return "Connected.";
		
							} else {
								
								if (!adminPassword.equals("")) return "jajidfm62mr7i8dtyfr2tyrypzea8tyyoejht3";
								else return "jajidfm62mr7i8dtyfr2tyrypzea8tyyoejht2";
								
							}
		
						default:
							
							//Script execution
				inputVector.remove(inputVector.size()-1);
				
				return StringFx.concat(session, inputVector, env);
							
						}
						
					}
		
				}
		
			case 4:
		
				switch (inputVector.get(0).value+" "+inputVector.get(1).value+" "+inputVector.get(2).value+" "+inputVector.get(3).value) {
				case "user first tab link":
					
					return UserManager.first(session.user);
		
				case "metric system nb processor":
					
					return Statistic.systemNbProcessor()+"";
		
				case "metric file system roots":
					
					return Statistic.fileSystemRoots();
		
				case "metric total swap mem":
					
					return Statistic.getTotalSwapSpaceSize()+"";
		
				case "metric current used mem":
					
					return Statistic.getUsedPhysicalMemorySize()+"";
		
				case "metric current free mem":
					
					return Statistic.getFreePhysicalMemorySize()+"";
		
				case "metric current cpu jvm":
					
					return Statistic.currentJvmCpuValue()+"";
		
				case "metric current mem jvm":
					
					return Statistic.currentMemJvm()+"";
		
				case "metric current cpu system":
					
					return Statistic.currentSystemCpuValue()+"";
		
				case "config session connect timeout":
					
					return ""+Start.SESSION_CONNECT_TIMEOUT;
		
				case "config web server port":
								
					return ""+Start.WEB_SERVER_PORT;
		
				case "config nb index file":
					
					return ""+Start.NB_INDEX_FILE;
		
				case "config data block size":
					
					return ""+DataFile.BLOCK_SIZE;
		
				case "user reset all tokens":

					UserManager.reset_tokens(session.user);
					
					return "Tokens generated with successful.";
					
				default:
					
					switch (inputVector.get(0).value+" "+inputVector.get(1).value+" "+inputVector.get(2).value) {
					case "relation list ids":
						
						String relationId = inputVector.get(3).value;
										
						return NodeManager.format(RelationManager.listTabLinks(session.user, relationId).toJSONString());
									
					case "fs index count":
						
						//Initialization
						int indexFileId = 0;
		
						//Check if the file id is valid
						try {
		
							indexFileId = Integer.parseInt(inputVector.get(3).value);
		
							if (indexFileId<0) {
		
								throw new Exception("err");
		
							}
		
						} catch (Exception e) {
		
							throw new Exception("Sorry, the file id is not valid.");
		
						}
		
						return ""+IndexFilePool.get(indexFileId).current_block_size();
		
					case "symbol show languages":
						
						String symbolName = inputVector.get(3).value;
						
						return JsonFormatter.format(SymbolManager.showLanguages(session.user, symbolName).toJSONString());
		
					case "stimulation search ra_rs":
						
						String fromUser = inputVector.get(3).value;
						
						if (!Brain.stimulation.containsKey(fromUser)) {
							
							throw new Exception("Sorry, the user '"+fromUser+"' has never initilalized his session.");
							
						}
						
						JSONArray stimulation_tab = (JSONArray) Brain.stimulation.get(fromUser);
						
						return StimulationManager.search_strategy(((JSONArray) stimulation_tab.get(stimulation_tab.size()-1)));
					
					case "relation check condition":
						
						fromUser = inputVector.get(3).value;
						
						if (!Brain.stimulation.containsKey(fromUser)) {
							
							throw new Exception("Sorry, the user '"+fromUser+"' has never initilalized his session.");
							
						}
						
						stimulation_tab = (JSONArray) Brain.stimulation.get(fromUser);
						
						return RelationManager.checkCondition(session, session.user, env, stimulation_tab, fromUser).toJSONString();
					
					case "relation execute action":
						
						fromUser = inputVector.get(3).value;
						
						if (!Brain.stimulation.containsKey(fromUser)) {
							
							throw new Exception("Sorry, the user '"+fromUser+"' has never initilalized his session.");
							
						}
						
						stimulation_tab = (JSONArray) Brain.stimulation.get(fromUser);
						
						return RelationManager.executeAction(session, session.user, env, fromUser);
					
					case "relation exist condition":
						
						relationId = inputVector.get(3).value;
						
						if (RelationManager.existCondition(session, session.user, relationId)) return "1";
						else return "0";
					
					case "stimulation search rl":
						
						fromUser = inputVector.get(3).value;
						
						if (!Brain.stimulation.containsKey(fromUser)) {
							
							throw new Exception("Sorry, the user '"+fromUser+"' has never initilalized his session.");
							
						}
						
						stimulation_tab = (JSONArray) Brain.stimulation.get(fromUser);
						
						return StimulationManager.search_logical(((JSONArray) stimulation_tab.get(stimulation_tab.size()-1)));
					
					case "stimulation create tab": 
						
						//Get parameters
						fromUser = inputVector.get(3).value;
						
						StimulationManager.createTab(fromUser);
						
						return "Tab created with successful.";

					case "is not null":
						
						//Get parameters
						String str = inputVector.get(3).value;
						
						return OperatorFx.is_not_null(str);
		
					case "json is loaded":
						
						String key = inputVector.get(3).value;
						
						return JsonManager.is_loaded(env, key);
		
					case "user get thought":
						
						String user = inputVector.get(3).value;
						
						return UserManager.getThought(user);
		
					case "relation generate condition":
						
						relationId = inputVector.get(3).value;
						
						return "j23i88m90m76i39t04r09y35p14a96y09e57t36"+RelationManager.generateConditions(relationId);
		
					case "relation generate action":
						
						relationId = inputVector.get(3).value;
						
						return "j23i88m90m76i39t04r09y35p14a96y09e57t36"+RelationManager.generateAction(relationId);
		
					case "relation show conditions":
						
						relationId = inputVector.get(3).value;
						
						return JsonFormatter.format(RelationManager.showConditions(session, session.user, relationId, env).toJSONString());
		
					case "relation show action":
						
						relationId = inputVector.get(3).value;
						
						return RelationManager.showAction(session, session.user, relationId, env);
		
					case "sequence generate update":
						
						key = inputVector.get(3).value;
						
						return SequenceManager.generateUpdate(key);
		
					case "parameter generate update":
						
						key = inputVector.get(3).value;
						
						return ParameterManager.generateUpdate(key);
		
					case "job generate update":
						
						String jobId = inputVector.get(3).value;
						
						return JobManager.generateUpdate(jobId);
		
					case "script generate url":
						
						String scriptName = inputVector.get(3).value;
						
						return ScriptManager.generateUrl(scriptName, session.user);
		
					case "script generate execute":
						
						scriptName = inputVector.get(3).value;
						
						return ScriptManager.generateExecute(scriptName);
		
					case "script generate include":
						
						scriptName = inputVector.get(3).value;
						
						return ScriptManager.generateInclude(scriptName);
		
					case "user show groups":
						
						String login = inputVector.get(3).value;
						
						return NodeManager.format(GroupManager.userGetGroup(login).toJSONString());
		
					case "relation show info":
						
						relationId = inputVector.get(3).value;
						
						return RelationManager.getInfo(session.user, relationId);
		
					case "relation show sentence":
						
						relationId = inputVector.get(3).value;
						
						return RelationManager.showSentence(session.user, relationId);
		
					case "user show scripts":
						
						login = inputVector.get(3).value;
						
						return NodeManager.format(GroupManager.userGetScript(login));
		
					case "script show groups":
						
						scriptName = inputVector.get(3).value;
						
						return NodeManager.format(GroupManager.scriptGetGroup(scriptName).toJSONString());
		
					case "script show users":
						
						scriptName = inputVector.get(3).value;
						
						return NodeManager.format(GroupManager.scriptGetUser(scriptName));
		
					case "group get user":
						
						String groupName = inputVector.get(3).value;
						
						return NodeManager.format(GroupManager.getUser(session.user, groupName));
		
					case "group get script":

						groupName = inputVector.get(3).value;

						return NodeManager.format(GroupManager.getScript(session.user, groupName));

					case "user from token":

						String token = inputVector.get(3).value;
						
						return UserManager.get_user_from_token(token);
						
					case "user reset token":

						user = inputVector.get(3).value;
						
						UserManager.reset_token(session.user, user);
						
						return "Token generated with successful.";
						
					case "link get value":

						key = inputVector.get(3).value;
						
						return LinkManager.get_value(key);
						
					case "is not empty":
						
						//Get parameters
						str = inputVector.get(3).value;
						
						return OperatorFx.is_not_empty(str);
		
					case "script generate update":
						
						String id = inputVector.get(3).value;
						
						return ScriptManager.generateUpdate(id);
					
					case "word show languages":
										
						//Get word
						String word = inputVector.get(3).value;
			
						return NodeManager.format(WordManager.showLanguages(session.user, word).toJSONString());
			
					case "word lang probability":
										
						//Get sentence
						String sentence = inputVector.get(3).value;
						
						Vector<Vector<MQLValue>> s = Misc.splitCommand(sentence);
						
						if (s.size()==0) {
							
							return new JSONArray().toJSONString();
							
						}
			
						return NodeManager.format(WordManager.getLangProbability(session.user, s.get(0)).toJSONString());
			
					case "relation thought stimulate":
						
						//Get thought id
						String thoughtId = inputVector.get(3).value;
						
						ThoughtManager.stimulateRelations(session.user, thoughtId, new JSONArray());
			
						return "Relations stimulated with successful.";
			
					case "parameter get value":
						
						//Get key
						key = inputVector.get(3).value;
		
						return ParameterManager.get_value(session.user, key);
		
					case "parameter get looked":
						
						//Get key
						key = inputVector.get(3).value;
		
						return ParameterManager.get_looked(session.user, key);
		
					case "sequence get current":
						
						//Get key
						key = inputVector.get(3).value;
		
						return SequenceManager.get_current(session.user, key);
		
					case "relation show thoughts":
						
						relationId = inputVector.get(3).value;
						
						return NodeManager.format(RelationManager.showThoughtsRecursively(session.user, relationId).toJSONString());
					
					case "relation show words":
						
						relationId = inputVector.get(3).value;
						
						return NodeManager.format(RelationManager.showWords(session.user, relationId).toJSONString());
					
					case "env del var":
						
						String var = inputVector.get(3).value;
						
						env.remove(var);
						
						return "Variable deleted with successful.";
					
					case "env get var":
						
						var = inputVector.get(3).value;
						
						return env.get(var);
					
					case "env exist var":
						
						var = inputVector.get(3).value;
						
						if (env.exist(var)) return "1";
						else return "0";
					
					case "thought show words":
										
						thoughtId = inputVector.get(3).value;
						
						return NodeManager.format(ThoughtManager.getWords(thoughtId).toJSONString());
					
					case "node show detailed":
						
						//Get keys
						key = inputVector.get(3).value;
		
						//SHOW RECORD
						return JsonFormatter.format(Record.getDetail(key).toJSONString());
		
					default:
		
						switch (inputVector.get(0).value+" "+inputVector.get(1).value) {
						case "word levenshtein":
							
							//Get string
							String symbolChars = inputVector.get(2).value;
							String order = inputVector.get(3).value;
							int iorder = 0;
							
							if (order.toLowerCase().equals("desc")) {
								
								iorder = 2;
								
							} else if (order.toLowerCase().equals("asc")) {
								
								iorder = 1;
								
							} else if (order.toLowerCase().equals("top")) {
								
								iorder = 0;
								
							} else {
								
								throw new Exception("Sorry, the order field is not valid (asc|desc|top).");
								
							}
		
							return WordManager.searchLevenshteinDistance(session.user, symbolChars, ConcentrationManager.getConcentrationDepth("C[symbol]"), iorder).toJSONString();
		
						case "relation translate":
							
							relationId = inputVector.get(2).value;
							String lang = inputVector.get(3).value;
							
							LanguageManager.exceptionIfNotExist(lang);
							
							return RelationManager.translate(session.user, relationId, lang);
			
						case "relation show":
							
							String thoughtIdOrRelationTabLinkId = inputVector.get(2).value;
							String type = inputVector.get(3).value;
							
							return NodeManager.format(RelationManager.showTabLinks(session.user, thoughtIdOrRelationTabLinkId, type, ConcentrationManager.getConcentrationDepth("C[relation]")).toJSONString());
						
						case "relation last":
							
							String relationTl = inputVector.get(2).value;
							type = inputVector.get(3).value;
							
							return RelationManager.lastTabLink(session.user, relationTl, type)+"";
			
						case "relation first":
							
							relationTl = inputVector.get(2).value;
							type = inputVector.get(3).value;
							
							return RelationManager.firstTabLink(session.user, relationTl, type)+"";
			
						case "word delete":
							
							//Get key
							word = inputVector.get(2).value;
							lang = inputVector.get(3).value;
							
							LanguageManager.exceptionIfNotExist(lang);
			
							WordManager.deleteLang(session.user, word, lang);
			
							return "Word W["+inputVector.get(2).value+"] deleted with successful in the language '"+lang+"'.";
			
						case "word perception":
							
							word = inputVector.get(2).value;
							lang = inputVector.get(3).value;
							
							LanguageManager.exceptionIfNotExist(lang);
							
							return JsonFormatter.format(WordManager.showTabLinkPerceptions(session.user, word, lang).toJSONString());
						
						case "symbol perception":
							
							String symbol = inputVector.get(2).value;
							lang = inputVector.get(3).value;
							
							LanguageManager.exceptionIfNotExist(lang);
							
							return JsonFormatter.format(SymbolManager.showPerceptionTabLinks(session.user, symbol, lang).toJSONString());
						
						case "word show":
							
							word = inputVector.get(2).value;
							lang = inputVector.get(3).value;
							
							LanguageManager.exceptionIfNotExist(lang);
							
							return JsonFormatter.format(WordManager.showTabLinks(session.user, word, lang).toJSONString());
			
						case "word first":
							
							word = inputVector.get(2).value;
							lang = inputVector.get(3).value;
							
							LanguageManager.exceptionIfNotExist(lang);
							
							return WordManager.firstTabLink(session.user, word, lang)+"";
			
						case "word last":
							
							word = inputVector.get(2).value;
							lang = inputVector.get(3).value;
							
							LanguageManager.exceptionIfNotExist(lang);
							
							return WordManager.lastTabLink(session.user, word, lang)+"";
			
						case "symbol first":
							
							symbol = inputVector.get(2).value;
							lang = inputVector.get(3).value;
							
							LanguageManager.exceptionIfNotExist(lang);
							
							return SymbolManager.firstTabLink(session.user, symbol, lang)+"";
			
						case "symbol last":
							
							symbol = inputVector.get(2).value;
							lang = inputVector.get(3).value;
							
							LanguageManager.exceptionIfNotExist(lang);
							
							return SymbolManager.lastTabLink(session.user, symbol, lang)+"";
			
						case "symbol show":
							
							symbol = inputVector.get(2).value;
							lang = inputVector.get(3).value;
							
							LanguageManager.exceptionIfNotExist(lang);
							
							return JsonFormatter.format(SymbolManager.showTabLinks(session.user, symbol, lang, ConcentrationManager.getConcentrationDepth("C[symbol]")).toJSONString());
			
						case "stimulation replace":
							
							fromUser = inputVector.get(2).value;
							
							String json = inputVector.get(3).value;
							
							if (!Brain.stimulation.containsKey(fromUser)) {
								
								throw new Exception("Sorry, the user '"+fromUser+"' has never initilalized his session.");
								
							}
							
							StimulationManager.set(fromUser, json);
							
							return "Stimulation replaced with successful.";
			
						case "stimulation get":
							
							fromUser = inputVector.get(2).value;
							
							String index = inputVector.get(3).value;
							
							if (!Brain.stimulation.containsKey(fromUser)) {
								
								throw new Exception("Sorry, the user '"+fromUser+"' has never initilalized his session.");
								
							}
							
							//Check if the number by page is valid
							try {
		
								Integer.parseInt(index);
		
							} catch (Exception e) {
		
								throw new Exception("Sorry, the number of line is not valid.");
		
							}
							
							return JsonFormatter.format(StimulationManager.get(Integer.parseInt(index), (JSONArray) Brain.stimulation.get(fromUser)).toJSONString());
			
						case "stimulation remove":
							
							fromUser = inputVector.get(2).value;
							
							index = inputVector.get(3).value;
							
							if (!Brain.stimulation.containsKey(fromUser)) {
								
								throw new Exception("Sorry, the user '"+fromUser+"' has never initilalized his session.");
								
							}
							
							//Check if the number by page is valid
							try {
		
								Integer.parseInt(index);
		
							} catch (Exception e) {
		
								throw new Exception("Sorry, the number of line is not valid.");
		
							}
							
							StimulationManager.remove(Integer.parseInt(index), (JSONArray) Brain.stimulation.get(fromUser));
			
							return "Stimulation removed with successful!";
							
						case "stimulation initialize":

							fromUser = inputVector.get(2).value;
							str = inputVector.get(3).value;
							
							try {
								return JsonFormatter.format(StimulationManager.call(true, fromUser, str, Brain.env, Brain.session));
							} catch (Exception e) {
								Transaction.rollback(Brain.user);
								throw e;
							}
			
						case "json load": 
							
							//Get parameters
							String docId = inputVector.get(2).value;
							String jsonString = inputVector.get(3).value;
							
							return JsonManager.load(env, docId, jsonString);
	
						case "user insert": case "user create": case "user add": 
							
							//Get parameters
							login = inputVector.get(2).value;
							String password = inputVector.get(3).value;
							
							UserManager.create(session.user, login, "", password, false);
	
							return "User "+login+" created with successful.";
	
						case "stimulation redirect":

							fromUser = inputVector.get(2).value;
							String data = inputVector.get(3).value;
							
							
							if (!Brain.stimulation.containsKey(fromUser)) {
								
								throw new Exception("Sorry, the user '"+fromUser+"' has never initilalized his session.");
								
							}
							
							StimulationManager.redirect(data, (JSONArray) Brain.stimulation.get(fromUser));
							
							return "Redirection executed with successfully.";
			
						case "file create":
							
							String filePath = inputVector.get(2).value;
							data = inputVector.get(3).value;
							
							return FileFx.create(filePath, data);
			
						case "string count":

							String string = inputVector.get(2).value;
							String find = inputVector.get(3).value;

							return StringFx.count( string, find)+"";

						case "string levenshtein_distance":

							String word1 = inputVector.get(2).value;
							String word2 = inputVector.get(3).value;

							return StringFx.levenshtein_distance(word1, word2)+"";

						case "string del_char_before_each_line":

							data = inputVector.get(2).value;
							String nbChar = inputVector.get(3).value;

							return StringFx.del_char_before_each_line( data, nbChar)+"";

						case "string ends_with":

							String stringValue = inputVector.get(2).value;
							String stringToEnd = inputVector.get(3).value;

							return StringFx.ends_with( stringValue, stringToEnd)+"";

						case "string indent":

							str = inputVector.get(2).value;
							String nbSpaceBefore = inputVector.get(3).value;

							return StringFx.indent( str, nbSpaceBefore)+"";

						case "string instr":

							String str1 = inputVector.get(2).value;
							String str2 = inputVector.get(3).value;

							return StringFx.instr( str1, str2)+"";

						case "string left":

							str = inputVector.get(2).value;
							String len = inputVector.get(3).value;

							return StringFx.left( str, len)+"";

						case "string like":

							str = inputVector.get(2).value;
							String pat = inputVector.get(3).value;

							return StringFx.like( str, pat)+"";

						case "string locate":

							str1 = inputVector.get(2).value;
							str2 = inputVector.get(3).value;

							return StringFx.locate( str1, str2)+"";

						case "string matches":

							str = inputVector.get(2).value;
							pat = inputVector.get(3).value;

							return StringFx.matches( str, pat)+"";

						case "string mid":

							str = inputVector.get(2).value;
							index = inputVector.get(3).value;

							return StringFx.mid( str, index)+"";

						case "string not_like":

							str = inputVector.get(2).value;
							pat = inputVector.get(3).value;

							return StringFx.not_like( str, pat)+"";

						case "string not_regexp":

							str = inputVector.get(2).value;
							pat = inputVector.get(3).value;

							return StringFx.not_regexp( str, pat)+"";

						case "string position":

							str1 = inputVector.get(2).value;
							str2 = inputVector.get(3).value;

							return StringFx.position( str1, str2)+"";

						case "string regexp":

							str = inputVector.get(2).value;
							pat = inputVector.get(3).value;

							return StringFx.regexp( str, pat)+"";

						case "string repeat":

							str = inputVector.get(2).value;
							String count = inputVector.get(3).value;

							return StringFx.repeat( str, count)+"";

						case "string right":

							str = inputVector.get(2).value;
							len = inputVector.get(3).value;

							return StringFx.right( str, len)+"";

						case "string starts_with":

							stringValue = inputVector.get(2).value;
							String stringToStart = inputVector.get(3).value;

							return StringFx.starts_with( stringValue, stringToStart)+"";

						case "string strcmp":

							str1 = inputVector.get(2).value;
							str2 = inputVector.get(3).value;

							return StringFx.strcmp( str1, str2)+"";

						case "string strpos":

							str1 = inputVector.get(2).value;
							str2 = inputVector.get(3).value;

							return StringFx.strpos( str1, str2)+"";

						case "string sublrchar":

							str = inputVector.get(2).value;
							String numberDeleteChar = inputVector.get(3).value;

							return StringFx.sublrchar( str, numberDeleteChar)+"";

						case "string substr":

							str = inputVector.get(2).value;
							index = inputVector.get(3).value;

							return StringFx.substr( str, index)+"";

						case "string substring":

							str = inputVector.get(2).value;
							index = inputVector.get(3).value;

							return StringFx.substring( str, index)+"";

						case "type is_char":

							String value = inputVector.get(2).value;
							String size = inputVector.get(3).value;

							return TypeFx.is_char( value, size)+"";

						case "type is_integer":

							value = inputVector.get(2).value;
							size = inputVector.get(3).value;

							return TypeFx.is_integer( value, size)+"";

						case "type is_valid_date":

							String dateToValidate = inputVector.get(2).value;
							String format = inputVector.get(3).value;

							return TypeFx.is_valid_date( dateToValidate, format)+"";

						case "type is_valid_timestamp":

							String timestampToValidate = inputVector.get(2).value;
							format = inputVector.get(3).value;

							return TypeFx.is_valid_timestamp( timestampToValidate, format)+"";

						case "type is_varchar":

							value = inputVector.get(2).value;
							size = inputVector.get(3).value;

							return TypeFx.is_varchar( value, size)+"";

						case "math atan2":

							String number1 = inputVector.get(2).value;
							String number2 = inputVector.get(3).value;

							return MathFx.atan2( number1, number2)+"";

						case "math bit_and":

							number1 = inputVector.get(2).value;
							number2 = inputVector.get(3).value;

							return MathFx.bit_and( number1, number2)+"";

						case "math bit_or":

							number1 = inputVector.get(2).value;
							number2 = inputVector.get(3).value;

							return MathFx.bit_or( number1, number2)+"";

						case "math bit_xor":

							number1 = inputVector.get(2).value;
							number2 = inputVector.get(3).value;

							return MathFx.bit_xor( number1, number2)+"";

						case "math hypot":

							number1 = inputVector.get(2).value;
							number2 = inputVector.get(3).value;

							return MathFx.hypot( number1, number2)+"";

						case "math max":

							number1 = inputVector.get(2).value;
							number2 = inputVector.get(3).value;

							return MathFx.max( number1, number2)+"";

						case "math min":

							number1 = inputVector.get(2).value;
							number2 = inputVector.get(3).value;

							return MathFx.min( number1, number2)+"";

						case "math mod":

							number1 = inputVector.get(2).value;
							number2 = inputVector.get(3).value;

							return MathFx.mod( number1, number2)+"";

						case "math pow":

							number1 = inputVector.get(2).value;
							number2 = inputVector.get(3).value;

							return MathFx.pow( number1, number2)+"";

						case "math power":

							number1 = inputVector.get(2).value;
							number2 = inputVector.get(3).value;

							return MathFx.power( number1, number2)+"";

						case "math round":

							number1 = inputVector.get(2).value;
							number2 = inputVector.get(3).value;

							return MathFx.round( number1, number2)+"";

						case "date is_valid_date":

							dateToValidate = inputVector.get(2).value;
							format = inputVector.get(3).value;

							return DateFx.is_valid_date( dateToValidate, format)+"";

						case "date is_valid_timestamp":

							timestampToValidate = inputVector.get(2).value;
							format = inputVector.get(3).value;

							return DateFx.is_valid_timestamp( timestampToValidate, format)+"";

						case "atom distinct":

							String atomList = inputVector.get(2).value;
							String separator = inputVector.get(3).value;

							return AtomFx.distinct( atomList, separator)+"";

						case "atom distinct_lrtrim":

							atomList = inputVector.get(2).value;
							separator = inputVector.get(3).value;

							return AtomFx.distinct_lrtrim( atomList, separator)+"";

						case "atom distinct_lrtrim_1sbefore":

							atomList = inputVector.get(2).value;
							separator = inputVector.get(3).value;

							return AtomFx.distinct_lrtrim_1sbefore( atomList, separator)+"";

						case "atom get_first":

							atomList = inputVector.get(2).value;
							separator = inputVector.get(3).value;

							return AtomFx.get_first( atomList, separator)+"";

						case "atom get_first_lrtrim":

							atomList = inputVector.get(2).value;
							separator = inputVector.get(3).value;

							return AtomFx.get_first_lrtrim( atomList, separator)+"";

						case "atom get_last":

							atomList = inputVector.get(2).value;
							separator = inputVector.get(3).value;

							return AtomFx.get_last( atomList, separator)+"";

						case "atom get_last_lrtrim":

							atomList = inputVector.get(2).value;
							separator = inputVector.get(3).value;

							return AtomFx.get_last_lrtrim( atomList, separator)+"";

						case "atom lrtrim":

							atomList = inputVector.get(2).value;
							separator = inputVector.get(3).value;

							return AtomFx.lrtrim( atomList, separator)+"";

						case "atom size":

							atomList = inputVector.get(2).value;
							separator = inputVector.get(3).value;

							return AtomFx.size( atomList, separator)+"";

						case "not equal":
							
							//Get parameters
							str1 = inputVector.get(2).value;
							str2 = inputVector.get(3).value;
							
							return OperatorFx.not_equal(str1, str2);
			
						case "node fields":
							
							//Get key, name and value
							key = inputVector.get(2).value;
							String jPath = inputVector.get(3).value;
		
							return JsonFormatter.format(NodeManager.fields(key, jPath));
		
						case "json fields":
							
							//Get key, name and value
							key = inputVector.get(2).value;
							jPath = inputVector.get(3).value;
		
							return JsonFormatter.format(JsonManager.fields(env, key, jPath));
		
						case "sequence add": case "sequence create": case "sequence insert":
							
							//Get key
							key = inputVector.get(2).value;
							value = inputVector.get(3).value;
		
							SequenceManager.add(session.user, key, value);
							
							return "Sequence added with successful.";
						
						case "sequence update":
							
							//Get key
							key = inputVector.get(2).value;
							value = inputVector.get(3).value;
						
							SequenceManager.update(session.user, key, value);
							
							return "Sequence updated with successful.";
						
						case "parameter update":
							
							//Get key
							key = inputVector.get(2).value;
							value = inputVector.get(3).value;
						
							ParameterManager.update(session.user, key, value);
							
							return "Parameter updated with successful.";
						
						case "symbol insert": case "symbol create": case "symbol add": 
							
							//Get key
							symbol = inputVector.get(2).value;
							lang = inputVector.get(3).value;
							
							LanguageManager.exceptionIfNotExist(lang);
		
							SymbolManager.create(session.user, symbol, lang);
		
							return "Symbol S["+inputVector.get(2).value+"] created with successful in the language '"+lang+"'.";
		
						case "symbol delete": 
											
							//Get key
							symbol = inputVector.get(2).value;
							lang = inputVector.get(3).value;
							
							LanguageManager.exceptionIfNotExist(lang);
						
							SymbolManager.delete(session.user, symbol, lang);
						
							return "Symbol S["+inputVector.get(2).value+"] deleted with successful in the language '"+lang+"'.";
						
						case "word stimulate":
							
							//Get word
							word = inputVector.get(2).value;
							lang = inputVector.get(3).value;
							
							LanguageManager.exceptionIfNotExist(lang);
		
							WordManager.stimulate(session.user, word, lang);
		
							return "Word W["+inputVector.get(2).value+"] stimulated with successful in the language '"+lang+"'.";
		
						case "thought show":
							
							//Get key
							word = inputVector.get(2).value;
							lang = inputVector.get(3).value;
							
							LanguageManager.exceptionIfNotExist(lang);
		
							return JsonFormatter.format(ThoughtManager.list(session.user, word, lang).toJSONString());
		
						case "thought merge":
							
							//Get keys
							String thoughtId1 = inputVector.get(2).value;
							String thoughtId2 = inputVector.get(3).value;
							
							ThoughtManager.merge(session.user, thoughtId1, thoughtId2);
		
							return "Thoughts merged with successful.";
		
						case "thought first":
							
							word = inputVector.get(2).value;
							lang = inputVector.get(3).value;
							
							LanguageManager.exceptionIfNotExist(lang);
							
							return WordManager.firstTabLink(session.user, word, lang)+"";
		
						case "thought last":
							
							word = inputVector.get(2).value;
							lang = inputVector.get(3).value;
							
							LanguageManager.exceptionIfNotExist(lang);
							
							return WordManager.lastTabLink(session.user, word, lang)+"";
		
						case "word insert": case "word create": case "word add": 
												
							//Get key
							key = inputVector.get(2).value;
							lang = inputVector.get(3).value;
							
							LanguageManager.exceptionIfNotExist(lang);
							
							return WordManager.create(session.user, key, "", lang);
		
						case "word exist":
							
							//Get key
							key = inputVector.get(2).value;
							lang = inputVector.get(3).value;
							
							LanguageManager.exceptionIfNotExist(lang);
							
							if(WordManager.exist(session.user, key, lang)) return "1";
							else return "0";
		
						default:
							
							switch (inputVector.get(0).value) {
							case "=>":
								lang = inputVector.get(1).value;
								key = inputVector.get(2).value;
								value = inputVector.get(3).value;
								
								LanguageManager.exceptionIfNotExist(lang);

								if (value.startsWith("TH")) {
									return LinkManager.create_thought(session.user, lang, key, value);
								} else {
									return LinkManager.create_relation(session.user, lang, key, value);
								}
							
							case "try":
								
								//Get parameters
								String mainCommand = inputVector.get(1).value;
								String commandIfError = inputVector.get(2).value;
								String exceptionId = inputVector.get(3).value;
				
								return Statement.try_mql(session, mainCommand, commandIfError, exceptionId, env);
				
							case "if":
								
								//Get parameters
								String condition = inputVector.get(1).value;
								String trueAction = inputVector.get(2).value;
								String falseAction = inputVector.get(3).value;
								
								return Statement.if_mql(session, condition, trueAction, falseAction, env);
				
							default:
								
								//Script execution
				inputVector.remove(inputVector.size()-1);
				
				return StringFx.concat(session, inputVector, env);
								
							}
							
						}
						
					}
					
				}
		
			case 5:
		
				switch (inputVector.get(0).value+" "+inputVector.get(1).value+" "+inputVector.get(2).value+" "+inputVector.get(3).value+" "+inputVector.get(4).value) {
				case "metric current used swap mem":
					
					return Statistic.getUsedSwapSpaceSize()+"";
		
				case "metric current free swap mem":
					
					return Statistic.getFreeSwapSpaceSize()+"";
		
				case "config nb data file open":
					
					return ""+DataFilePool.NB_DATA_FILE_OPEN;
		
				case "config nb index file open":
					
					return ""+IndexFilePool.NB_INDEX_FILE_OPEN;
		
				case "config max data block size":
					
					return ""+DataFile.MAX_BLOCK;
		
				default:
					
					switch (inputVector.get(0).value+" "+inputVector.get(1).value+" "+inputVector.get(2).value+" "+inputVector.get(3).value) {
					case "relation pos no something":
						
						String relationId = inputVector.get(4).value;
						
						return RelationManager.firstNotSomethingPosition(session.user, relationId)+"";
					
					case "fs unused last block":
						
						int fileId = 0;
						
						//Check if the file id is valid
						try {
			
							fileId = Integer.parseInt(inputVector.get(4).value);
			
							if (fileId<0) {
			
								throw new Exception("err");
			
							}
			
						} catch (Exception e) {
			
							throw new Exception("Sorry, the file id is not valid.");
			
						}
			
						return ""+DataFilePool.get(fileId).unusedFile.last_unused_block();
			
					case "fs unused nb block":
						
						fileId = 0;
						
						//Check if the file id is valid
						try {
			
							fileId = Integer.parseInt(inputVector.get(4).value);
			
							if (fileId<0) {
			
								throw new Exception("err");
			
							}
			
						} catch (Exception e) {
			
							throw new Exception("Sorry, the file id is not valid.");
			
						}
			
						return ""+DataFilePool.get(fileId).unusedFile.nb_unused_block();
			
					case "is null or empty":
						
						//Get parameters
						String str = inputVector.get(4).value;
						
						return OperatorFx.is_null_or_empty(str);
		
					case "relation show thought nodes":
						
						relationId = inputVector.get(4).value;
						
						return NodeManager.format(RelationManager.showThoughtNodes(session.user, relationId).toJSONString());
					
					case "fs index calcul pos":
						
						//Get keys
						String key = inputVector.get(4).value;
						
						JSONObject obj = new JSONObject();
						
						//Get the index file position
						int indexFileToWrite = IndexFile.indexFileId(key);
						
						//Get the position into the index file tab
						long indexFileTabPosition = IndexFile.indexFileTabIndex(key, IndexFilePool.get(indexFileToWrite).current_max_block_size());
		
						obj.put("key", key);
						obj.put("indexFileId", indexFileToWrite);
						obj.put("indexPosition", indexFileTabPosition*12+8);
		
						//SHOW RECORD
						return JsonFormatter.format(obj.toJSONString());
						
					case "user get tab link":
						
						int position = 0;
					
						//Check if the depth is valid
						try {
					
							position = Integer.parseInt(inputVector.get(4).value);
					
							if (position<0) {
					
								throw new Exception("err");
					
							}
					
						} catch (Exception e) {
					
							throw new Exception("Sorry, the position is not valid.");
					
						}
						
						return UserManager.get(session.user, position);
					
					default:
						
						switch (inputVector.get(0).value+" "+inputVector.get(1).value+" "+inputVector.get(2).value) {
						case "stimulation search basic":
							
							String fromUser = inputVector.get(3).value;
							String typeList = inputVector.get(4).value;
							
							if (!Brain.stimulation.containsKey(fromUser)) {
								
								throw new Exception("Sorry, the user '"+fromUser+"' has never initilalized his session.");
								
							}
							
							JSONArray stimulation_tab = (JSONArray) Brain.stimulation.get(fromUser);
							
							return JsonFormatter.format(StimulationManager.basic_search(((JSONArray) stimulation_tab.get(stimulation_tab.size()-1)), typeList).toJSONString());
			
						case "relation search basic":
							
							//Get key
							String text = inputVector.get(3).value;
							typeList = inputVector.get(4).value;
			
							return JsonFormatter.format(BasicSearchEngine.execute(session.user, text, typeList).toJSONString());
			
						case "relation show perceptions":
							
							relationId = inputVector.get(3).value;
							String type = inputVector.get(4).value;
							
							return JsonFormatter.format(RelationManager.showTabLinkPerceptions(session.user, relationId, type).toJSONString());
						
						case "relation has sub":
							
							relationId = inputVector.get(3).value;
							type = inputVector.get(4).value;
							
							if (RelationManager.containsTabLink(session.user, relationId, type)) return "1";
							else return "0";
						
						case "fs data block":
							
							//Initialization
							long pos = 0;
							fileId = 0;
			
							//Check if the file id is valid
							try {
			
								fileId = Integer.parseInt(inputVector.get(3).value);
			
								if (fileId<0) {
			
									throw new Exception("err");
			
								}
			
							} catch (Exception e) {
			
								throw new Exception("Sorry, the file id is not valid.");
			
							}
			
							//Check if the position is valid
							try {
			
								pos = Long.parseLong(inputVector.get(4).value);
			
								if (pos<0) {
			
									throw new Exception("err");
			
								}
			
							} catch (Exception e) {
			
								throw new Exception("Sorry, the position is not valid.");
			
							}
			
							return Block.read(DataFilePool.get(fileId), pos).toString();
			
						case "fs data record":
							
							//Initialization
							pos = 0;
							fileId = 0;
			
							//Check if the file id is valid
							try {
			
								fileId = Integer.parseInt(inputVector.get(3).value);
			
								if (fileId<0) {
			
									throw new Exception("err");
			
								}
			
							} catch (Exception e) {
			
								throw new Exception("Sorry, the file id is not valid.");
			
							}
			
							//Check if the position is valid
							try {
			
								pos = Long.parseLong(inputVector.get(4).value);
			
								if (pos<0) {
			
									throw new Exception("err");
			
								}
			
							} catch (Exception e) {
			
								throw new Exception("Sorry, the position is not valid.");
			
							}
			
							JSONArray result = new JSONArray();
							String recString = Record.read(DataFilePool.get(fileId), 1, pos, result);
							result.add(Block.decodeb64(recString));
			
							return JsonFormatter.format(result.toJSONString());
			
						case "fs index block":
							
							//Initialization
							pos = 0;
							fileId = 0;
			
							//Check if the file id is valid
							try {
			
								fileId = Integer.parseInt(inputVector.get(3).value);
			
								if (fileId<0) {
			
									throw new Exception("err");
			
								}
			
							} catch (Exception e) {
			
								throw new Exception("Sorry, the file id is not valid.");
			
							}
			
							//Check if the position is valid
							try {
			
								pos = Long.parseLong(inputVector.get(4).value);
			
								if (pos<0) {
			
									throw new Exception("err");
			
								}
			
							} catch (Exception e) {
			
								throw new Exception("Sorry, the position is not valid.");
			
							}
			
							return IndexFile.toString(fileId, pos);
			
						case "word perception thought":
							
							String word = inputVector.get(3).value;
							String lang = inputVector.get(4).value;
							
							LanguageManager.exceptionIfNotExist(lang);
							
							return JsonFormatter.format(WordManager.showThoughtPerceptions(session.user, word, lang).toJSONString());
						
						case "word perception symbol":
							
							word = inputVector.get(3).value;
							lang = inputVector.get(4).value;
							
							LanguageManager.exceptionIfNotExist(lang);
							
							return JsonFormatter.format(WordManager.showSymbolPerception(session.user, word, lang).toJSONString());
						
						case "user force insert": case "user force create": case "user force add": 
							
							//Get parameters
							String login = inputVector.get(3).value;
							String password = inputVector.get(4).value;
							
							UserManager.create(session.user, login, "", password, true);
	
							return "User "+login+" created with successful.";
	
						case "group grant user":
							
							login = inputVector.get(3).value;
							String groupName = inputVector.get(4).value;
							
							GroupManager.grantUser(session.user, groupName, login);
			
							return "User granted with successful.";
			
						case "group grant script":
							
							String scriptId = inputVector.get(3).value;
							groupName = inputVector.get(4).value;
							
							GroupManager.grantScript(session.user, groupName, scriptId);
			
							return "Script granted with successful.";
			
						case "group ungrant user":
							
							login = inputVector.get(3).value;
							groupName = inputVector.get(4).value;
							
							GroupManager.ungrantUser(session.user, groupName, login);
			
							return "User ungranted with successful.";
			
						case "group ungrant script":
							
							scriptId = inputVector.get(3).value;
							groupName = inputVector.get(4).value;
							
							GroupManager.ungrantScript(session.user, groupName, scriptId);
			
							return "Script ungranted with successful.";
			
						case "script is granted":

							String scriptName = inputVector.get(3).value;
							login = inputVector.get(4).value;
							
							if (ScriptManager.isGrantedToUser(session.user, login, scriptName)) return "1";
							else return "0";
							
						case "link delete thought":

							lang = inputVector.get(3).value;
							key = inputVector.get(4).value;
							
							LanguageManager.exceptionIfNotExist(lang);
							
							LinkManager.delete_thought(session.user, lang, key);

							return "Link deleted with successful.";
							
						case "link delete relation":

							lang = inputVector.get(3).value;
							key = inputVector.get(4).value;
							
							LanguageManager.exceptionIfNotExist(lang);
							
							LinkManager.delete_relation(session.user, lang, key);

							return "Link deleted with successful.";
							
						case "link get thought":

							lang = inputVector.get(3).value;
							key = inputVector.get(4).value;
							
							LanguageManager.exceptionIfNotExist(lang);
							
							return LinkManager.get_thought(lang, key);
							
						case "link get relation":

							lang = inputVector.get(3).value;
							key = inputVector.get(4).value;
							
							LanguageManager.exceptionIfNotExist(lang);
							
							return LinkManager.get_relation(lang, key);
							
						case "link exist thought":

							lang = inputVector.get(3).value;
							key = inputVector.get(4).value;
							
							LanguageManager.exceptionIfNotExist(lang);
							
							if (LinkManager.exist_thought(lang, key)) return "1";
							else return "1";
							
						case "link exist relation":

							lang = inputVector.get(3).value;
							key = inputVector.get(4).value;
							
							LanguageManager.exceptionIfNotExist(lang);
							
							if (LinkManager.exist_relation(lang, key)) return "1";
							else return "1";
							
						case "relation set action":
							
							relationId = inputVector.get(3).value;
							String action = inputVector.get(4).value;
							
							RelationManager.setAction(session, session.user, relationId, action, env);
			
							return "Action inserted with successful.";
			
						case "thought show words":
												
							String thoughtId = inputVector.get(3).value;
							lang = inputVector.get(4).value;
							
							LanguageManager.exceptionIfNotExist(lang);
							
							return NodeManager.format(ThoughtManager.getWords(thoughtId, lang).toJSONString());
						
						case "env set var":
							
							String var = inputVector.get(3).value;
							String val = inputVector.get(4).value;
							
							env.set(var, val);
							
							return val;
						
						case "env incr var":
							
							var = inputVector.get(3).value;
							val = inputVector.get(4).value;
							
							return env.incr(var, val);
						
						case "env decr var":
							
							var = inputVector.get(3).value;
							val = inputVector.get(4).value;
							
							return env.decr(var, val);
						
						case "concentration set depth":
							
							key = inputVector.get(3).value;
							int depth = 0;
						
							//Check if the depth is valid
							try {
						
								depth = Integer.parseInt(inputVector.get(4).value);
						
								if (depth<=0) {
						
									throw new Exception("err");
						
								}
						
							} catch (Exception e) {
						
								throw new Exception("Sorry, the depth value is not valid.");
						
							}
							
							ConcentrationManager.setConcentrationDepth(session.user, key, depth);
							
							return "Concentration depth saved with successful.";
						
						case "fs data size":
							
							//Initialization
							int page = 0;
							int nbByPage = 100;
		
							//Check if the page is valid
							try {
		
								page = Integer.parseInt(inputVector.get(3).value)-1;
		
								if (page<0) {
		
									throw new Exception("err");
		
								}
		
							} catch (Exception e) {
		
								throw new Exception("Sorry, the page is not valid.");
		
							}
		
							//Check if the number by page is valid
							try {
		
								nbByPage = Integer.parseInt(inputVector.get(4).value);
		
								if (nbByPage<0) {
		
									throw new Exception("err");
		
								}
		
							} catch (Exception e) {
		
								throw new Exception("Sorry, the number by page is not valid.");
		
							}
		
							//Parse all data files
							int index = page*nbByPage;
							int counter = nbByPage;
							JSONArray jsonResult = new JSONArray();
							while (true) {
		
								if (counter==0) {
		
									break;
		
								}
		
								//Check if the file does not exist
								if (!(new File("data"+File.separator+"data"+File.separator+index+File.separator+"brain"+index+".dat")).exists()) {
		
									break;
		
								}
		
								//Get the data file
								DataFile dataFile = DataFilePool.get(index);
		
								JSONObject sess = new JSONObject();
								sess.put("dataFileId", index);
								sess.put("percent_graph", dataFile.per30_used());
								sess.put("current_block_size", dataFile.current_block_size());
								sess.put("current_max_block_size", dataFile.current_max_block_size());
								sess.put("percent_used", dataFile.percent_used());
								sess.put("size", dataFile.get_size());
								sess.put("lastUnusedBlock", dataFile.unusedFile.last_unused_block());
								sess.put("nbUnusedBlock", dataFile.unusedFile.nb_unused_block());
								
								jsonResult.add(sess);
								
								index++;
								counter--;
		
							}
							
							return JsonFormatter.format(jsonResult.toJSONString());
		
						case "fs index size":
							
							//Initialization
							page = 0;
							nbByPage = 100;
		
							//Check if the page is valid
							try {
		
								page = Integer.parseInt(inputVector.get(3).value)-1;
		
								if (page<0) {
		
									throw new Exception("err");
		
								}
		
							} catch (Exception e) {
		
								throw new Exception("Sorry, the page is not valid.");
		
							}
		
							//Check if the number by page is valid
							try {
		
								nbByPage = Integer.parseInt(inputVector.get(4).value);
		
								if (nbByPage<0) {
		
									throw new Exception("err");
		
								}
		
							} catch (Exception e) {
		
								throw new Exception("Sorry, the number by page is not valid.");
		
							}
		
							//Parse all data files
							index = page*nbByPage;
							counter = nbByPage;
							jsonResult = new JSONArray();
							while (true) {
		
								if (counter==0) {
		
									break;
		
								}
		
								//Check if the file does not exist
								if (!(new File("data"+File.separator+"index"+File.separator+index+File.separator+"brain"+index+".idx")).exists()) {
		
									break;
		
								}
		
								//Get the index file
								IndexFile indexFile = IndexFilePool.get(index);
		
								JSONObject sess = new JSONObject();
								sess.put("indexFileId", index);
								sess.put("percent_graph", indexFile.per30_used());
								sess.put("current_block_size", indexFile.current_block_size());
								sess.put("current_max_block_size", indexFile.current_max_block_size());
								sess.put("percent_used", indexFile.percent_used());
								sess.put("size", indexFile.get_size());
								
								jsonResult.add(sess);
								
								index++;
								counter--;
		
							}
							
							return JsonFormatter.format(jsonResult.toJSONString());
		
						case "node select element":
								
							//Get key, name and value
							key = inputVector.get(3).value;
							String xPath = inputVector.get(4).value;
			
							return NodeManager.select(key, xPath);
		
						case "json select element":
							
							//Get key, name and value
							key = inputVector.get(3).value;
							xPath = inputVector.get(4).value;
			
							return JsonManager.select(env, key, xPath);
		
						case "node is object":
								
							//Get key, name and value
							key = inputVector.get(3).value;
							xPath = inputVector.get(4).value;
			
							return NodeManager.is_obj(key, xPath);
		
						case "node is array":
								
							//Get key, name and value
							key = inputVector.get(3).value;
							xPath = inputVector.get(4).value;
			
							return NodeManager.is_array(key, xPath);
			
						case "json is object":
							
							//Get key, name and value
							key = inputVector.get(3).value;
							xPath = inputVector.get(4).value;
			
							return JsonManager.is_obj(env, key, xPath);
		
						case "json is array":
								
							//Get key, name and value
							key = inputVector.get(3).value;
							xPath = inputVector.get(4).value;
			
							return JsonManager.is_array(env, key, xPath);
			
						case "node count element":
								
							//Get key, name and value
							key = inputVector.get(3).value;
							xPath = inputVector.get(4).value;
			
							return NodeManager.count(key, xPath);
		
						case "json count element":
							
							//Get key, name and value
							key = inputVector.get(3).value;
							xPath = inputVector.get(4).value;
			
							return JsonManager.count(env, key, xPath);
		
						case "symbol show words":
							
							String symbol = inputVector.get(3).value;
							lang = inputVector.get(4).value;
							
							LanguageManager.exceptionIfNotExist(lang);
							
							return JsonFormatter.format(SymbolManager.showWords(session.user, symbol, lang));
		
						case "user set password":
							
							//Get user info
							String user = inputVector.get(3).value;
							String pwd = inputVector.get(4).value;
		
							UserManager.updatePassword(session.user, user, pwd);
							
							return "Password updated with successful.";
						
						case "metric index files":
							
							//Initialization
							page = 0;
							nbByPage = 100;
		
							//Check if the page is valid
							try {
		
								page = Integer.parseInt(inputVector.get(3).value)-1;
		
								if (page<0) {
		
									throw new Exception("err");
		
								}
		
							} catch (Exception e) {
		
								throw new Exception("Sorry, the page is not valid.");
		
							}
		
							//Check if the number by page is valid
							try {
		
								nbByPage = Integer.parseInt(inputVector.get(4).value);
		
								if (nbByPage<0) {
		
									throw new Exception("err");
		
								}
		
							} catch (Exception e) {
		
								throw new Exception("Sorry, the number by page is not valid.");
		
							}
		
							//Parse all data files
							index = page*nbByPage;
							counter = nbByPage;
							String r = "";
							while (true) {
		
								if (counter==0) {
		
									break;
		
								}
		
								//Check if the file does not exist
								if (!(new File("data"+File.separator+"index"+File.separator+index+File.separator+"brain"+index+".idx")).exists()) {
		
									break;
		
								}
		
								//Get the index file
								IndexFile indexFile = IndexFilePool.get(index);
								
								if (r.equals("")) {
									r += "Total block: "+Start.NB_INDEX_FILE+"x"+indexFile.current_max_block_size()+"="+(Start.NB_INDEX_FILE*indexFile.current_max_block_size())+"\n";
									r += "Total size: "+Start.NB_INDEX_FILE+"x"+indexFile.get_size()+"="+(Start.NB_INDEX_FILE*indexFile.get_size())+"b\n";
								}
		
								r += indexFile.per30_used()+" "+indexFile.percent_used()+"%";
								r += " - "+index+" - used: "+indexFile.current_block_size();
								r += ", free: "+(indexFile.current_max_block_size()-indexFile.current_block_size());
								r += ", total block: "+indexFile.current_max_block_size();
								r += ", size: "+indexFile.get_size()+"b";
								r += "\n";
								
								index++;
								counter--;
		
							}
							
							return r;
		
						case "metric data files":
							
							//Initialization
							page = 0;
							nbByPage = 100;
		
							//Check if the page is valid
							try {
		
								page = Integer.parseInt(inputVector.get(3).value)-1;
		
								if (page<0) {
		
									throw new Exception("err");
		
								}
		
							} catch (Exception e) {
		
								throw new Exception("Sorry, the page is not valid.");
		
							}
		
							//Check if the number by page is valid
							try {
		
								nbByPage = Integer.parseInt(inputVector.get(4).value);
		
								if (nbByPage<0) {
		
									throw new Exception("err");
		
								}
		
							} catch (Exception e) {
		
								throw new Exception("Sorry, the number by page is not valid.");
		
							}
		
							//Parse all data files
							index = page*nbByPage;
							counter = nbByPage;
							r = "";
							while (true) {
		
								if (counter==0) {
		
									break;
		
								}
								
								//Check if the file does not exist
								if (!(new File("data"+File.separator+"data"+File.separator+index+File.separator+"brain"+index+".dat")).exists()) {
		
									break;
		
								}
		
								//Get the data file
								DataFile dataFile = DataFilePool.get(index);
		
								r += dataFile.per30_used()+" "+dataFile.percent_used()+"%";
								r += " - "+index+" - used: "+dataFile.current_block_size();
								r += ", free: "+(dataFile.current_max_block_size()-dataFile.current_block_size());
								r += ", total block: "+dataFile.current_max_block_size();
								r += ", size: "+dataFile.get_size()+"b";
								r += ", max size: "+((dataFile.current_max_block_size()*dataFile.get_size())/dataFile.current_block_size()/1000000)+"Mb";
								r += ", last unused block: "+dataFile.unusedFile.last_unused_block();
								r += ", nb unused block: "+dataFile.unusedFile.nb_unused_block();
								r += "\n";
								
								index++;
								counter--;
		
							}
							
							return r;
		
						default:
							
							switch (inputVector.get(0).value+" "+inputVector.get(1).value) {
							case "relation new": 
								
								//Get sentence
								String sentence = inputVector.get(2).value;
								type = inputVector.get(3).value;
								lang = inputVector.get(4).value;
								
								LanguageManager.exceptionIfNotExist(lang);
					
								return RelationManager.createFromSentence(session, env, session.user, sentence, type, lang);
					
							case "relation get":
								
								String relationTabLink = inputVector.get(2).value;
								int positionInt = 0;
				
								//Check if the file id is valid
								try {
				
									positionInt = Integer.parseInt(inputVector.get(3).value);
				
									if (positionInt<0) {
				
										throw new Exception("err");
				
									}
				
								} catch (Exception e) {
				
									throw new Exception("Sorry, the position is not valid.");
				
								}
								type = inputVector.get(4).value;
								
								return RelationManager.getTabLink(session.user, relationTabLink, positionInt, type);
				
							case "relation find":
								
								relationId = inputVector.get(2).value;
								positionInt = 0;
				
								//Check if the file id is valid
								try {
				
									positionInt = Integer.parseInt(inputVector.get(3).value);
				
									if (positionInt<0) {
				
										throw new Exception("err");
				
									}
				
								} catch (Exception e) {
				
									throw new Exception("Sorry, the position is not valid.");
				
								}
								type = inputVector.get(4).value;
								
								return RelationManager.findTabLink(session.user, relationId, positionInt, type);
				
							case "word levenshtein":
								
								//Get string
								String symbolChars = inputVector.get(2).value;
								String order = inputVector.get(3).value;
								lang = inputVector.get(4).value;
								int iorder = 0;
								
								LanguageManager.exceptionIfNotExist(lang);
								
								if (order.toLowerCase().equals("desc")) {
									
									iorder = 2;
									
								} else if (order.toLowerCase().equals("asc")) {
									
									iorder = 1;
									
								} else if (order.toLowerCase().equals("top")) {
									
									iorder = 0;
									
								} else {
									
									throw new Exception("Sorry, the order field is not valid (asc|desc|top).");
									
								}
			
								return WordManager.searchLevenshteinDistance(session.user, symbolChars, ConcentrationManager.getConcentrationDepth("C[symbol]"), iorder, lang).toJSONString();
			
							case "word get":
								
								word = inputVector.get(2).value;
								lang = inputVector.get(4).value;
								position = 0;
								
								LanguageManager.exceptionIfNotExist(lang);
				
								//Check if the file id is valid
								try {
				
									position = Integer.parseInt(inputVector.get(3).value);
				
									if (position<0) {
				
										throw new Exception("err");
				
									}
				
								} catch (Exception e) {
				
									throw new Exception("Sorry, the position is not valid.");
				
								}
								
								return WordManager.getTabLink(session.user, word, position, lang);
				
							case "symbol get":
								
								symbol = inputVector.get(2).value;
								lang = inputVector.get(4).value;
								position = 0;
								
								LanguageManager.exceptionIfNotExist(lang);
				
								//Check if the file id is valid
								try {
				
									position = Integer.parseInt(inputVector.get(3).value);
				
									if (position<0) {
				
										throw new Exception("err");
				
									}
				
								} catch (Exception e) {
				
									throw new Exception("Sorry, the position is not valid.");
				
								}
								
								return SymbolManager.getTabLink(session.user, symbol, position, lang);
				
							case "word insert": case "word create": case "word add": 
								
								//Get key
								key = inputVector.get(2).value;
								lang = inputVector.get(3).value;
								String separator = inputVector.get(4).value;
								
								LanguageManager.exceptionIfNotExist(lang);
								
								return WordManager.create(session.user, key, separator, lang);
			
							case "rest http":

								String method = inputVector.get(2).value;
								String beginUrl = inputVector.get(3).value;
								String endUrl = inputVector.get(4).value;

								return REST.http_exe(method, beginUrl, endUrl);

							case "string split":

								str = inputVector.get(2).value;
								String regex = inputVector.get(3).value;
								String limit = inputVector.get(4).value;

								return StringFx.split(str, regex, limit);

							case "rest https":

								method = inputVector.get(2).value;
								beginUrl = inputVector.get(3).value;
								endUrl = inputVector.get(4).value;

								return REST.https_exe(method, beginUrl, endUrl);

							case "string instr":

								String str1 = inputVector.get(2).value;
								String str2 = inputVector.get(3).value;
								String fromIndex = inputVector.get(4).value;

								return StringFx.instr( str1, str2, fromIndex)+"";

							case "string locate":

								str1 = inputVector.get(2).value;
								str2 = inputVector.get(3).value;
								fromIndex = inputVector.get(4).value;

								return StringFx.locate( str1, str2, fromIndex)+"";

							case "string lpad":

								str = inputVector.get(2).value;
								String padString = inputVector.get(3).value;
								String paddedLength = inputVector.get(4).value;

								return StringFx.lpad( str, padString, paddedLength)+"";

							case "string mid":

								str = inputVector.get(2).value;
								String beginIndex = inputVector.get(3).value;
								String endIndex = inputVector.get(4).value;

								return StringFx.mid( str, beginIndex, endIndex)+"";

							case "string position":

								str1 = inputVector.get(2).value;
								str2 = inputVector.get(3).value;
								fromIndex = inputVector.get(4).value;

								return StringFx.position( str1, str2, fromIndex)+"";

							case "string repeat_insert_str":

								str = inputVector.get(2).value;
								String strToInsert = inputVector.get(3).value;
								String incr = inputVector.get(4).value;

								return StringFx.repeat_insert_str( str, strToInsert, incr)+"";

							case "string replace":

								str = inputVector.get(2).value;
								String target = inputVector.get(3).value;
								String replacement = inputVector.get(4).value;

								return StringFx.replace( str, target, replacement)+"";

							case "string rpad":

								str = inputVector.get(2).value;
								padString = inputVector.get(3).value;
								paddedLength = inputVector.get(4).value;

								return StringFx.rpad( str, padString, paddedLength)+"";

							case "string strpos":

								str1 = inputVector.get(2).value;
								str2 = inputVector.get(3).value;
								fromIndex = inputVector.get(4).value;

								return StringFx.strpos( str1, str2, fromIndex)+"";

							case "string substr":

								str = inputVector.get(2).value;
								beginIndex = inputVector.get(3).value;
								endIndex = inputVector.get(4).value;

								return StringFx.substr( str, beginIndex, endIndex)+"";

							case "string substring":

								str = inputVector.get(2).value;
								beginIndex = inputVector.get(3).value;
								endIndex = inputVector.get(4).value;

								return StringFx.substring( str, beginIndex, endIndex)+"";

							case "string encode":

								str = inputVector.get(2).value;
								String sourceEnc = inputVector.get(3).value;
								String destinationEnc = inputVector.get(4).value;

								return StringFx.encode( str, sourceEnc, destinationEnc)+"";

							case "type is_bool":

								String value = inputVector.get(2).value;
								String bool1 = inputVector.get(3).value;
								String bool2 = inputVector.get(4).value;

								return TypeFx.is_bool(session, value, bool1, bool2, env)+"";

							case "type is_decimal":

								String stringDecimal = inputVector.get(2).value;
								String digitBeforeTheDecimalPoint = inputVector.get(3).value;
								String digitAfterTheDecimalPoint = inputVector.get(4).value;

								return TypeFx.is_decimal( stringDecimal, digitBeforeTheDecimalPoint, digitAfterTheDecimalPoint)+"";

							case "date add":

								String date = inputVector.get(2).value;
								String field = inputVector.get(3).value;
								String number = inputVector.get(4).value;

								return DateFx.add( date, field, number)+"";

							case "date addt":

								String timestamp = inputVector.get(2).value;
								field = inputVector.get(3).value;
								number = inputVector.get(4).value;

								return DateFx.addt( timestamp, field, number)+"";

							case "date dateadd":

								date = inputVector.get(2).value;
								field = inputVector.get(3).value;
								number = inputVector.get(4).value;

								return DateFx.dateadd( date, field, number)+"";

							case "date dateaddt":

								timestamp = inputVector.get(2).value;
								field = inputVector.get(3).value;
								number = inputVector.get(4).value;

								return DateFx.dateaddt( timestamp, field, number)+"";

							case "date datediff":

								date = inputVector.get(2).value;
								field = inputVector.get(3).value;
								number = inputVector.get(4).value;

								return DateFx.datediff( date, field, number)+"";

							case "date datedifft":

								timestamp = inputVector.get(2).value;
								field = inputVector.get(3).value;
								number = inputVector.get(4).value;

								return DateFx.datedifft( timestamp, field, number)+"";

							case "date diff":

								date = inputVector.get(2).value;
								field = inputVector.get(3).value;
								number = inputVector.get(4).value;

								return DateFx.diff( date, field, number)+"";

							case "date difft":

								timestamp = inputVector.get(2).value;
								field = inputVector.get(3).value;
								number = inputVector.get(4).value;

								return DateFx.difft( timestamp, field, number)+"";

							case "date format":

								String timestampToFormat = inputVector.get(2).value;
								String formatIn = inputVector.get(3).value;
								String formatOut = inputVector.get(4).value;

								return DateFx.format( timestampToFormat, formatIn, formatOut)+"";

							case "atom before_exclud":

								String atomList = inputVector.get(2).value;
								String index_str = inputVector.get(3).value;
								separator = inputVector.get(4).value;

								return AtomFx.before_exclud( atomList, index_str, separator)+"";

							case "atom before_includ":

								atomList = inputVector.get(2).value;
								index_str = inputVector.get(3).value;
								separator = inputVector.get(4).value;

								return AtomFx.before_includ( atomList, index_str, separator)+"";

							case "atom count":

								atomList = inputVector.get(2).value;
								String atomToCount = inputVector.get(3).value;
								separator = inputVector.get(4).value;

								return AtomFx.count( atomList, atomToCount, separator)+"";

							case "atom count_distinct":

								atomList = inputVector.get(2).value;
								atomToCount = inputVector.get(3).value;
								separator = inputVector.get(4).value;

								return AtomFx.count_distinct( atomList, atomToCount, separator)+"";

							case "atom count_lrtrim":

								atomList = inputVector.get(2).value;
								atomToCount = inputVector.get(3).value;
								separator = inputVector.get(4).value;

								return AtomFx.count_lrtrim( atomList, atomToCount, separator)+"";

							case "atom count_lrtrim_distinct":

								atomList = inputVector.get(2).value;
								atomToCount = inputVector.get(3).value;
								separator = inputVector.get(4).value;

								return AtomFx.count_lrtrim_distinct( atomList, atomToCount, separator)+"";

							case "atom find":

								atomList = inputVector.get(2).value;
								String atomToFind = inputVector.get(3).value;
								separator = inputVector.get(4).value;

								return AtomFx.find( atomList, atomToFind, separator)+"";

							case "atom find_lrtrim":

								atomList = inputVector.get(2).value;
								atomToFind = inputVector.get(3).value;
								separator = inputVector.get(4).value;

								return AtomFx.find_lrtrim( atomList, atomToFind, separator)+"";

							case "atom get":

								atomList = inputVector.get(2).value;
								index_str = inputVector.get(3).value;
								separator = inputVector.get(4).value;

								return AtomFx.get( atomList, index_str, separator)+"";

							case "atom get_lrtrim":

								atomList = inputVector.get(2).value;
								index_str = inputVector.get(3).value;
								separator = inputVector.get(4).value;

								return AtomFx.get_lrtrim( atomList, index_str, separator)+"";

							case "atom position":

								atomList = inputVector.get(2).value;
								atomToFind = inputVector.get(3).value;
								separator = inputVector.get(4).value;

								return AtomFx.position( atomList, atomToFind, separator)+"";

							case "atom position_lrtrim":

								atomList = inputVector.get(2).value;
								atomToFind = inputVector.get(3).value;
								separator = inputVector.get(4).value;

								return AtomFx.position_lrtrim( atomList, atomToFind, separator)+"";

							case "if force":
								
								//Get parameters
								String condition = inputVector.get(2).value;
								String trueAction = inputVector.get(3).value;
								String falseAction = inputVector.get(4).value;
								
								return Statement.if_force_mql(session, condition, trueAction, falseAction, env);
				
							case "job add": case "job create": case "job insert":
								
								//Get parameters
								String jobId = inputVector.get(2).value;
								scriptName = inputVector.get(3).value;
								String patten = inputVector.get(4).value;
			
								JobManager.add(session.user, jobId, scriptName, patten);
								
								return "Job added with successful.";
							
							case "job update":
								
								//Get parameters
								jobId = inputVector.get(2).value;
								scriptName = inputVector.get(3).value;
								patten = inputVector.get(4).value;
			
								JobManager.update(session.user, jobId, scriptName, patten);
								
								return "Job updated with successful.";
							
							case "relation insert": case "relation create": case "relation add":
								
								//Get thoughts
								String thoughts = inputVector.get(2).value;
								type = inputVector.get(3).value;
								lang = inputVector.get(4).value;
								
								LanguageManager.exceptionIfNotExist(lang);
								
								return RelationManager.create(session.user, thoughts, lang, type);
				
							case "thought insert": case "thought create":  case "thought add": 
								
								//Get key
								word = inputVector.get(2).value;
								separator = inputVector.get(3).value;
								lang = inputVector.get(4).value;
								
								LanguageManager.exceptionIfNotExist(lang);
			
								return ThoughtManager.create(session.user, word, separator, lang);
			
							case "parameter add": case "parameter create": case "parameter insert":
								
								//Get key
								key = inputVector.get(2).value;
								value = inputVector.get(3).value;
								String looked = inputVector.get(4).value;
								
								if (!looked.equals("1") && !looked.equals("0")) {
									
									throw new Exception("Sorry, the looked field must be a boolean (1 or 0).");
									
								}
		
								ParameterManager.add(session.user, key, value, looked);
								
								return "Parameter added with successful.";
							
							case "thought get":
								
								word = inputVector.get(2).value;
								lang = inputVector.get(4).value;
								positionInt = 0;
								
								LanguageManager.exceptionIfNotExist(lang);
		
								//Check if the file id is valid
								try {
		
									positionInt = Integer.parseInt(inputVector.get(3).value);
		
									if (positionInt<0) {
		
										throw new Exception("err");
		
									}
		
								} catch (Exception e) {
		
									throw new Exception("Sorry, the position is not valid.");
		
								}
								
								return WordManager.getTabLink(session.user, word, positionInt, lang);
		
							case "user insert": case "user create": case "user add": 
								
								//Get parameters
								login = inputVector.get(2).value;
								password = inputVector.get(3).value;
								separator = inputVector.get(4).value;
								
								UserManager.create(session.user, login, separator, password, false);
		
								return "User "+login+" created with successful.";
		
							default: 
								
								switch (inputVector.get(0).value) {
									
								case "for":
									
									//Get parameters
									String init = inputVector.get(1).value;
									condition = inputVector.get(2).value;
									String increment = inputVector.get(3).value;
									action = inputVector.get(4).value;
					
									return Statement.for_mql(session, init, condition, increment, action, env);
					
								default:
									
									//Script execution
				inputVector.remove(inputVector.size()-1);
				
				return StringFx.concat(session, inputVector, env);
									
								}
								
							}
							
						}
						
					}
					
				}
		
			case 6:
				
				switch (inputVector.get(0).value+" "+inputVector.get(1).value+" "+inputVector.get(2).value+" "+inputVector.get(3).value+" "+inputVector.get(4).value) {
				default :
		
					switch (inputVector.get(0).value+" "+inputVector.get(1).value+" "+inputVector.get(2).value+" "+inputVector.get(3).value) {
					case "fs unused data block":
						
						//Initialization
						int limit = 0;
						int fileId = 0;
			
						//Check if the file id is valid
						try {
			
							fileId = Integer.parseInt(inputVector.get(4).value);
			
							if (fileId<0) {
			
								throw new Exception("err");
			
							}
			
						} catch (Exception e) {
			
							throw new Exception("Sorry, the file id is not valid.");
			
						}
			
						//Check if the limit is valid
						try {
			
							limit = Integer.parseInt(inputVector.get(5).value);
			
							if (limit<0) {
			
								throw new Exception("err");
			
							}
			
						} catch (Exception e) {
			
							throw new Exception("Sorry, the limit is not valid.");
			
						}
			
						return UnusedFile.list(DataFilePool.get(fileId), limit);
			
					case "group is granted user":
						
						String login = inputVector.get(4).value;
						String groupName = inputVector.get(5).value;
						
						if (GroupManager.isGrantedUser(session.user, groupName, login)) return "1";
						return "0";
		
					case "group is granted script":
						
						String scriptId = inputVector.get(4).value;
						groupName = inputVector.get(5).value;
						
						if (GroupManager.isGrantedScript(session.user, groupName, scriptId)) return "1";
						else return "0";
		
					case "relation set else action":
						
						String relationTabLinkId = inputVector.get(4).value;
						String elseAction = inputVector.get(5).value;
						
						RelationManager.setElseAction(session, session.user, relationTabLinkId, elseAction, env);
		
						return "Else action inserted with successful.";
		
					case "thought delete by word":
						
						//Get key
						String word = inputVector.get(4).value;
						String thoughtId = inputVector.get(5).value;
		
						ThoughtManager.deleteWord(session.user, thoughtId, word);
		
						return "Thought "+thoughtId+" deleted with successful for the word '"+word+"'.";
		
					case "thought delete by lang":
						
						//Get key
						String lang = inputVector.get(4).value;
						thoughtId = inputVector.get(5).value;
						
						LanguageManager.exceptionIfNotExist(lang);
		
						ThoughtManager.deleteLang(session.user, thoughtId, lang);
		
						return "Thought "+thoughtId+" deleted with successful in the language '"+lang+"'.";
		
					default:
						
						switch (inputVector.get(0).value+" "+inputVector.get(1).value+" "+inputVector.get(2).value) {
						case "json load select": 
							
							//Get parameters
							String docId = inputVector.get(3).value;
							String jsonString = inputVector.get(4).value;
							String jPath = inputVector.get(5).value;
							
							return JsonManager.load(env, docId, jsonString, jPath);
	
						case "user force insert": case "user force create": case "user force add": 
							
							//Get parameters
							login = inputVector.get(3).value;
							String password = inputVector.get(4).value;
							String separator = inputVector.get(5).value;
							
							UserManager.create(session.user, login, separator, password, true);
	
							return "User "+login+" created with successful.";
	
						case "relation set condition":
							
							String relationId = inputVector.get(3).value;
							String target = inputVector.get(4).value;
							String condition = inputVector.get(5).value;
							
							RelationManager.setCondition(session, session.user, relationId, target, condition, env);
			
							return "Condition inserted with successful.";
			
						case "brain send message":
							
							String fromUser = inputVector.get(3).value;
							String type = inputVector.get(4).value;
							String msg = inputVector.get(5).value;
							
							int iType = 0;
							
							//Check if the file id is valid
							try {
				
								iType = Integer.parseInt(type);
				
							} catch (Exception e) {
				
								throw new Exception("Sorry, the type is not a number.");
				
							}
							
							Session.aiSendMessageToMe(fromUser, iType, 
									msg
									);
			
							return "1";
			
						case "link create thought": case "link insert thought": case "link add thought":
							
							lang = inputVector.get(3).value;
							String key = inputVector.get(4).value;
							thoughtId = inputVector.get(5).value;
							
							LanguageManager.exceptionIfNotExist(lang);
							
							return LinkManager.create_thought(session.user, lang, key, thoughtId);
							
						case "link create relation": case "link insert relation": case "link add relation":

							lang = inputVector.get(3).value;
							key = inputVector.get(4).value;
							relationId = inputVector.get(5).value;
							
							LanguageManager.exceptionIfNotExist(lang);
							
							return LinkManager.create_relation(session.user, lang, key, relationId);
							
						default: 
						
							switch (inputVector.get(0).value+" "+inputVector.get(1).value) {
							case "circle merge": 

								String relationId1 = inputVector.get(2).value;
								String positions1 = inputVector.get(3).value;
								String relationId2 = inputVector.get(4).value;
								String positions2 = inputVector.get(5).value;
								
								return CircleManager.mergeCircle(session.user, relationId1, positions1, relationId2, positions2);
							
							case "math decimal_format":
	
								String number = inputVector.get(2).value;
								String pattern = inputVector.get(3).value;
								String decimalSeparator = inputVector.get(4).value;
								String groupingSeparator = inputVector.get(5).value;
	
								return MathFx.decimal_format( number, pattern, decimalSeparator, groupingSeparator)+"";
	
							/*case "emotion insert": case "emotion create": case "emotion add": 
								
								//Get parameters
								String emotion = inputVector.get(2).value;
								String theType = inputVector.get(3).value;
								type = inputVector.get(4).value;
								lang = inputVector.get(5).value;
								
								if (theType==null || (!theType.equals("1") && !theType.equals("0") && !theType.equals("2"))) {
									
									throw new Exception("Sorry, the level option is not valid (1, 0 or 2).");
									
								}
								
								EmotionManager.create(session.user, emotion, lang, Integer.parseInt(theType), type, false, null, 0);
				
								return "Language "+lang+" created with successful.";
								*/
							case "script add": case "script create": case "script insert":
								
								String scriptName = inputVector.get(2).value;
								String inMem = inputVector.get(3).value;
								String variables = inputVector.get(4).value;
								String mql = inputVector.get(5).value;
								
								if (!inMem.equals("1") && !inMem.equals("0")) {
									
									throw new Exception("Sorry, the inMem field must be a boolean (1 or 0).");
									
								}
								
								ScriptManager.add(session.user, scriptName, variables, mql, inMem);
								GroupManager.grantScript(session.user, "all", scriptName);
								
								return "Script added with successful.";
							
							case "script update":
								
								scriptName = inputVector.get(2).value;
								inMem = inputVector.get(3).value;
								variables = inputVector.get(4).value;
								mql = inputVector.get(5).value;
								
								if (!inMem.equals("1") && !inMem.equals("0")) {
									
									throw new Exception("Sorry, the inMem field must be a boolean (1 or 0).");
									
								}
								
								ScriptManager.update(session.user, scriptName, variables, mql, inMem);
								
								return "Script updated with successful.";
							
							case "word search":
								
								//Get string
								String symbolChars = inputVector.get(2).value;
								String regex = inputVector.get(3).value;
								String order = inputVector.get(4).value;
								lang = inputVector.get(5).value;
								int iorder = 0;
								
								LanguageManager.exceptionIfNotExist(lang);
								
								if (order.toLowerCase().equals("desc")) {
									
									iorder = 2;
									
								} else if (order.toLowerCase().equals("asc")) {
									
									iorder = 1;
									
								} else if (order.toLowerCase().equals("top")) {
									
									iorder = 0;
									
								} else {
									
									throw new Exception("Sorry, the order field is not valid (asc|desc|top).");
									
								}
		
								return WordManager.search(session.user, symbolChars, regex, ConcentrationManager.getConcentrationDepth("C[symbol]"), iorder, lang).toJSONString();
		
							default:
								
								//Script execution
				inputVector.remove(inputVector.size()-1);
				
				return StringFx.concat(session, inputVector, env);
								
							}
							
						}
						
					}
					
				}
		
			case 7:
				
				switch (inputVector.get(0).value+" "+inputVector.get(1).value+" "+inputVector.get(2).value+" "+inputVector.get(3).value+" "+inputVector.get(4).value) {
				default:
					
					switch (inputVector.get(0).value+" "+inputVector.get(1).value+" "+inputVector.get(2).value+" "+inputVector.get(3).value) {
					case "json load by ref": 
						
						//Get parameters
						String fromDocId = inputVector.get(4).value;
						String jPath = inputVector.get(5).value;
						String newDocId = inputVector.get(6).value;
						
						return JsonManager.loadByRef(env, fromDocId, newDocId, jPath);

					case "node delete from array":
						
						//Get key, name and value
						String key = inputVector.get(4).value;
						String xPath = inputVector.get(5).value;
						String index = inputVector.get(6).value;
		
						NodeManager.adelete(session.user, key, xPath, index);
		
						return "Deleted with successful.";
		
					case "node delete from object":
						
						//Get key, name and value
						key = inputVector.get(4).value;
						xPath = inputVector.get(5).value;
						String fieldName = inputVector.get(6).value;
		
						NodeManager.odelete(session.user, key, xPath, fieldName);
		
						return "Deleted with successful.";
		
					case "json delete from array":
						
						//Get key, name and value
						key = inputVector.get(4).value;
						xPath = inputVector.get(5).value;
						index = inputVector.get(6).value;
		
						return JsonManager.adelete(env, key, xPath, index);
		
					case "json delete from object":
						
						//Get key, name and value
						key = inputVector.get(4).value;
						xPath = inputVector.get(5).value;
						fieldName = inputVector.get(6).value;
		
						return JsonManager.odelete(env, key, xPath, fieldName);
		
					default:
		
						//Script execution
				inputVector.remove(inputVector.size()-1);
				
				return StringFx.concat(session, inputVector, env);
						
					}
					
				}
				
			case 8:
				
				switch (inputVector.get(0).value+" "+inputVector.get(1).value+" "+inputVector.get(2).value+" "+inputVector.get(3).value+" "+inputVector.get(4).value) {
				case "thought delete by word lang":
					
					//Get key
					String word = inputVector.get(5).value;
					String lang = inputVector.get(6).value;
					String thoughtId = inputVector.get(7).value;
					
					LanguageManager.exceptionIfNotExist(lang);
		
					ThoughtManager.delete(session.user, word, thoughtId, lang);
		
					return "Thought "+thoughtId+" deleted with successful for the word '"+word+"' and the language '"+lang+"'.";
		
				default :
		
					switch (inputVector.get(0).value+" "+inputVector.get(1).value+" "+inputVector.get(2).value+" "+inputVector.get(3).value) {
					case "word import from text":
							
						//Get parameters
						String text = inputVector.get(4).value;
						lang = inputVector.get(5).value;
						String separator = inputVector.get(6).value;
						String excludedChars = inputVector.get(7).value;
						
						LanguageManager.exceptionIfNotExist(lang);
			
						WordManager.import_from_text(session.user, text, separator, lang, excludedChars);
			
						return "Words imported with successful.";
			
					case "node insert into array":
						
						//Get key, name and value
						String key = inputVector.get(4).value;
						String xPath = inputVector.get(5).value;
						String value = inputVector.get(6).value;
						String type = inputVector.get(7).value;
			
						NodeManager.ainsert(session.user, key, xPath, value, type);
			
						return "Inserted with successful.";
			
					case "json insert into array":
						
						//Get key, name and value
						key = inputVector.get(4).value;
						xPath = inputVector.get(5).value;
						value = inputVector.get(6).value;
						type = inputVector.get(7).value;
			
						return JsonManager.ainsert(env, key, xPath, value, type);
			
					default:
						
						switch (inputVector.get(0).value+" "+inputVector.get(1).value+" "+inputVector.get(2).value) {
						case "relation search deep":
							
							String json = inputVector.get(3).value;
							String typeFilter = inputVector.get(4).value;
							String typeOrder = inputVector.get(5).value;
							String nbResult = inputVector.get(6).value;
							String str_depth = inputVector.get(7).value;
							int d = 0, nb = 0;
							JSONArray relations = null;
							
							try {
								
								relations = (JSONArray) JsonManager.load(json);
								
							} catch (Exception e) {
								
								throw new Exception("Sorry, the json object is not a valid array.");
								
							}
							
							try {
								
								nb = Integer.parseInt(nbResult);
								
							} catch (Exception e) {
								
								throw new Exception("Sorry, the number of result is not a valid number.");
								
							}
							
							try {
								
								d = Integer.parseInt(str_depth);
								
							} catch (Exception e) {
								
								throw new Exception("Sorry, the depth is not a valid number.");
								
							}
							
							return NodeManager.format(DeepSearchEngine.execute_from_mql(relations, typeFilter, typeOrder, nb, d).toJSONString());
						
						case "relation insert sub": case "relation create sub":  case "relation add sub": 
							
							//Get relation id and thoughts
							String relationId = inputVector.get(3).value;
							String thoughts = inputVector.get(4).value;
							String positions = inputVector.get(5).value;
							type = inputVector.get(6).value;
							lang = inputVector.get(7).value;
							
							LanguageManager.exceptionIfNotExist(lang);
							
							return RelationManager.createSubWithPosition(session.user, relationId, thoughts, positions, lang, type);
	
						default:
						
							if (inputVector.get(0).value.equals("node") && inputVector.get(1).value.equals("update") && 
								inputVector.get(2).value.equals("array")) {
								
								//Get key, name and value
								key = inputVector.get(3).value;
								xPath = inputVector.get(4).value;
								String index = inputVector.get(5).value;
								value = inputVector.get(6).value;
								type = inputVector.get(7).value;
								
								NodeManager.aupdate(session.user, key, xPath, index, value, type);
				
								return "Updated with successful.";
				
							} else if (inputVector.get(0).value.equals("node") && inputVector.get(1).value.equals("update") && 
									inputVector.get(2).value.equals("object")) {
								
								//Get key, name and value
								key = inputVector.get(3).value;
								xPath = inputVector.get(4).value;
								String fieldName = inputVector.get(5).value;
								value = inputVector.get(6).value;
								type = inputVector.get(7).value;
				
								NodeManager.oupdate(session.user, key, xPath, fieldName, value, type);
				
								return "Updated with successful.";
				
							} else if (inputVector.get(0).value.equals("json") && inputVector.get(1).value.equals("update") && 
								inputVector.get(2).value.equals("array")) {
							
								//Get key, name and value
								key = inputVector.get(3).value;
								xPath = inputVector.get(4).value;
								String index = inputVector.get(5).value;
								value = inputVector.get(6).value;
								type = inputVector.get(7).value;
								
								return JsonManager.aupdate(env, key, xPath, index, value, type);
				
							} else if (inputVector.get(0).value.equals("json") && inputVector.get(1).value.equals("update") && 
									inputVector.get(2).value.equals("object")) {
								
								//Get key, name and value
								key = inputVector.get(3).value;
								xPath = inputVector.get(4).value;
								String fieldName = inputVector.get(5).value;
								value = inputVector.get(6).value;
								type = inputVector.get(7).value;
				
								return JsonManager.oupdate(env, key, xPath, fieldName, value, type);
				
							} else {
								
								//Script execution
								inputVector.remove(inputVector.size()-1);
				
								return StringFx.concat(session, inputVector, env);
								
							}
							
						}
						
					}
				}
		
			case 9:
		
				switch (inputVector.get(0).value+" "+inputVector.get(1).value+" "+inputVector.get(2).value+" "+inputVector.get(3).value) {
				case "node insert into object":
					
					//Get key, name and value
					String key = inputVector.get(4).value;
					String xPath = inputVector.get(5).value;
					String fieldName = inputVector.get(6).value;
					String value = inputVector.get(7).value;
					String type = inputVector.get(8).value;
	
					NodeManager.oinsert(session.user, key, xPath, fieldName, value, type);
	
					return "Inserted with successful.";
	
				case "json insert into object":
					
					//Get key, name and value
					key = inputVector.get(4).value;
					xPath = inputVector.get(5).value;
					fieldName = inputVector.get(6).value;
					value = inputVector.get(7).value;
					type = inputVector.get(8).value;
	
					return JsonManager.oinsert(env, key, xPath, fieldName, value, type);
	
				case "json insert into array":
					
					//Get key, name and value
					key = inputVector.get(4).value;
					xPath = inputVector.get(5).value;
					String index = inputVector.get(6).value;
					value = inputVector.get(7).value;
					type = inputVector.get(8).value;
		
					return JsonManager.ainsert(env, key, xPath, index, value, type);
		
				case "node insert into array":
					
					//Get key, name and value
					key = inputVector.get(4).value;
					xPath = inputVector.get(5).value;
					index = inputVector.get(6).value;
					value = inputVector.get(7).value;
					type = inputVector.get(8).value;
		
					NodeManager.ainsert(session.user, key, xPath, index, value, type);
					
					return "Inserted with successful.";
		
				case "json set into array":
					
					//Get key, name and value
					key = inputVector.get(4).value;
					xPath = inputVector.get(5).value;
					index = inputVector.get(6).value;
					value = inputVector.get(7).value;
					type = inputVector.get(8).value;
				
					return JsonManager.aset(env, key, xPath, index, value, type);
				
				default:
					
					//Script execution
			inputVector.remove(inputVector.size()-1);
			
			return StringFx.concat(session, inputVector, env);
					
				}
		
			default:
		
				//Script execution
				inputVector.remove(inputVector.size()-1);
				
				return StringFx.concat(session, inputVector, env);
		
			}
			
		}
	
	}

}
