/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.core.log;

import java.io.File;
import java.io.FileWriter;
import java.text.SimpleDateFormat;

import javax.swing.JOptionPane;

import re.jpayet.mentdb.core.stat.Statistic;

//The log class
public class Log {

	//The current ID of file log writer (mentdb.logX); X=0 or 1 or 2 or ... or 9
	public static int currentLogFileId = 0;

	//Default max log file size = 30 Mb
	public static long maxLogSize = 30;

	//Default number of line to show
	public static int currentNbLine = 10;
	
	//Show the last log text added into the log system
	public static String show(int nbLine) {
		
		//Set the current number of line if 0
		if (nbLine == 0) {
			
			nbLine = currentNbLine; 
		
		} else {
			
			currentNbLine = nbLine;
			
		}
		
		if (Statistic.systemName().toLowerCase().indexOf("windows")>-1) {
			nbLine = nbLine*2;
		}
		
		//Initialization
		int fileId = currentLogFileId;
		String lastLine = "";

		//Initialization
		java.io.RandomAccessFile fileHandler = null;
		
		try {
			
			//Initialization
			fileHandler = new java.io.RandomAccessFile( new File("logs"+File.separator+"mentdb"+fileId+".log"), "r" );
			
			long fileLength = fileHandler.length() - 1;
			StringBuilder sb = new StringBuilder();
			int line = 0;

			//Parse the file
			for(long filePointer = fileLength; filePointer != -1; filePointer--){
				fileHandler.seek( filePointer );
				int readByte = fileHandler.readByte();

				//Break if end of line
				if( readByte == 0xA ) {
					line = line + 1;
					if (line == nbLine) {
						if (filePointer == fileLength) {
							continue;
						}
						break;
					}
				} else if( readByte == 0xD ) {
					line = line + 1;
					if (line == nbLine) {
						if (filePointer == fileLength - 1) {
							continue;
						}
						break;
					}
				}
				
				//Append data
				sb.append( ( char ) readByte );
				
			}

			//Reverse the string
			lastLine = sb.reverse().toString();
			
		} catch( java.io.IOException e ) {
			
		} finally {
			
			if (fileHandler != null )
				
				try {
					
					fileHandler.close();
					
				} catch (Exception e) {
			}
			
		}
		
		//Return the line
		return lastLine;
		
	}
	
	/**
	 * @name write
	 * @description Write a trace in the log file system
	 */
	public static void write(String message) {

		//Initialization
		FileWriter fileWriterCurrentLogFile = null;

		//Prepare the current log file
		File currentLogFile = new File("logs"+File.separator+"mentdb"+currentLogFileId+".log");

		try {

			//Check if current log file size is too large
			if (currentLogFile.length()>=(maxLogSize)*1000000)
			{

				//Too large then write in next log file
				currentLogFileId+=1;
				if (currentLogFileId>9) currentLogFileId=0;
				currentLogFile = new File("logs"+File.separator+"mentdb"+currentLogFileId+".log");

				//Clear the next log file
				currentLogFile.delete();
				currentLogFile.createNewFile();

			}

			//Write your message
			fileWriterCurrentLogFile = new FileWriter(currentLogFile, true);
			fileWriterCurrentLogFile.write((new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SZ")).format(System.currentTimeMillis())+": "+message+System.getProperty("line.separator"));

		}
		catch (Exception e) {
			
			JOptionPane.showMessageDialog(null, (new File(".")).getAbsolutePath()+"Err: "+e.getMessage());
			
		} finally {

			try {fileWriterCurrentLogFile.flush();} catch (Exception e) {};
			try {fileWriterCurrentLogFile.close();} catch (Exception e) {};

		};
		
	}

}
