/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.core.stat;

import java.io.File;
import java.lang.management.ManagementFactory;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;

import com.sun.management.OperatingSystemMXBean;

import re.jpayet.mentdb.tools.Misc;

//The server threads
@SuppressWarnings("restriction")
public class Statistic extends Thread {
	
	//Statistic
	public static Vector<String> stat_date = new Vector<String>();
	public static JSONArray stat_tot_session = new JSONArray();
	public static JSONArray stat_tot_command = new JSONArray();
	public static JSONArray stat_cpu = new JSONArray();
	public static JSONArray stat_mem = new JSONArray();
	public static String currentDate = "";
	public static Timer timer24s = new Timer();
	public static Timer timer10m = new Timer();
	public static OperatingSystemMXBean bean = (com.sun.management.OperatingSystemMXBean) ManagementFactory
    .getOperatingSystemMXBean();
	
	@SuppressWarnings("unchecked")
	public static String getStatSession() throws ParseException {

		JSONArray result = new JSONArray();
		
		for(int i=0;i<stat_date.size();i++) {
			
			JSONArray obj = new JSONArray();
			obj.add(new SimpleDateFormat("yyyy-MM-dd").parse((String) stat_date.get(i)).getTime());
			obj.add(stat_tot_session.get(i));
			result.add(obj);
			
		}
		
		return result.toJSONString();

	}

	static class RemindTask5s extends TimerTask {
		@SuppressWarnings("unchecked")
		public void run() {
			
			JSONArray tmpCpu = new JSONArray();
			tmpCpu.add((new Date()).getTime());
			tmpCpu.add(currentSystemCpuValue());
			stat_cpu.add(tmpCpu);
			
			if (stat_cpu.size()>2880) {
				stat_cpu.remove(0);
			}
			
			
			JSONArray tmpMem = new JSONArray();
			tmpMem.add((new Date()).getTime());
			tmpMem.add((int) (getUsedPhysicalMemorySize()*100)/getTotalPhysicalMemorySize());
			stat_mem.add(tmpMem);
			
			if (stat_mem.size()>2880) {
				stat_mem.remove(0);
			}
			
		}
	}

	static class RemindTask24h extends TimerTask {
		public void run() {
			
			currentDate = initCurrentDate();
			addNewDay();
			deleteOldDay();
			
		}
	}

	//Stop the scheduler
	public static void stopHeartbeat10m() {

	    timer10m.cancel();
	    timer10m.purge();;

	}

	//Stop the scheduler
	public static void stopHeartbeat24h() {

		timer24s.cancel();
		timer24s.purge();;

	}

	//Start the scheduler
	public static void startHeartbeat10m() {

	    timer10m.schedule(new RemindTask5s(), 0, 600 * 1000);

	}

	//Start the scheduler
	public static void startHeartbeat24h() {
		
		currentDate = initCurrentDate();
		addNewDay();
		deleteOldDay();
		
		Calendar today = Calendar.getInstance();
		today.set(Calendar.HOUR_OF_DAY, 24);
		today.set(Calendar.MINUTE, 0);
		today.set(Calendar.SECOND, 0);

		timer24s.schedule(new RemindTask24h(), today.getTime(), 86400000);

	}

	@SuppressWarnings("unchecked")
	public static synchronized void incrementNbSession() {
		
		stat_tot_session.set(stat_tot_session.size()-1, (Integer) stat_tot_session.get(stat_tot_session.size()-1)+1);
		
	}

	@SuppressWarnings("unchecked")
	public static void incrementNbCommand() {
		
		stat_tot_command.set(stat_tot_command.size()-1, (Integer) stat_tot_command.get(stat_tot_command.size()-1)+1);
		
	}

	public static int countCmd() {
		
		int c = 0;
		for(int i=0;i<stat_tot_command.size();i++) {
			
			c += (Integer) stat_tot_command.get(i);
			
		}
		
		return c;
		
	}
	
	public static String currentMemJvm() {
		
		//Initialization
		String result = "";
		
		Runtime runtime = Runtime.getRuntime();
		
		result = ((runtime.totalMemory() - runtime.freeMemory())/(1024*1024))+"/"+(runtime.totalMemory()/(1024*1024))+"/"+(runtime.maxMemory()/(1024*1024));
		
		return result;
		
	}

	public static int countSession() {
		
		int c = 0;
		for(int i=0;i<stat_tot_session.size();i++) {
			
			c += (Integer) stat_tot_session.get(i);
			
		}
		
		return c;
		
	}

	public static String initCurrentDate() {

		//Get the current day number
		Date dt = new Date();
		return new SimpleDateFormat("yyyy-MM-dd").format(dt);

	}

	//Amount of free swap memory in bytes.
	public static long getUsedSwapSpaceSize() {

		return (bean.getTotalSwapSpaceSize()-bean.getFreeSwapSpaceSize())/(1024*1024);

	}

	//Get the system name
	public static String systemName() {

		return bean.getName();

	}

	//Get the system architecture
	public static String systemArchitecture() {

		return bean.getArch();

	}

	//Get the system version
	public static String systemVersion() {

		return bean.getVersion();

	}

	//Get the system nb processor
	public static int systemNbProcessor() {

		return bean.getAvailableProcessors();

	}

	//Get the java vendor
	public static String systemJavaVendor() {

		return System.getProperty("java.vendor");

	}

	//Get the java version
	public static String systemJavaVersion() {

		return System.getProperty("java.version");

	}

	//Get the file system roots
	@SuppressWarnings("unchecked")
	public static String fileSystemRoots() {
		
		JSONArray all = new JSONArray();
		
		/* Get a list of all filesystem roots on this system */
	    File[] roots = File.listRoots();

	    /* For each filesystem root, print some info */
	    for (File root : roots) {
	    	
	    	JSONObject rootObj = new JSONObject();

	    	rootObj.put("absolutePath", ""+root.getAbsolutePath());
	    	rootObj.put("totalSpace", ""+root.getTotalSpace()/(1024*1024));
	    	rootObj.put("freeSpace", ""+root.getFreeSpace()/(1024*1024));
	    	rootObj.put("usedSpace", ""+(root.getTotalSpace()-root.getFreeSpace())/(1024*1024));
	    	
	    	if (root.getTotalSpace()>0) all.add(rootObj);
	        
	    }

		return all.toJSONString();

	}

	//Amount of free swap memory in bytes.
	public static long getFreeSwapSpaceSize() {

		return (bean.getFreeSwapSpaceSize()/(1024*1024));

	}

	//Total amount of swap memory in bytes.
	public static long getTotalSwapSpaceSize() {

		return (bean.getTotalSwapSpaceSize()/(1024*1024));

	}

	//Amount of free physical memory in bytes.
	public static long getUsedPhysicalMemorySize() {

		return (bean.getTotalPhysicalMemorySize()-bean.getFreePhysicalMemorySize())/(1024*1024);

	}

	//Amount of free physical memory in bytes.
	public static long getFreePhysicalMemorySize() {

		return (bean.getFreePhysicalMemorySize()/(1024*1024));

	}

	//Total amount of physical memory in bytes.
	public static long getTotalPhysicalMemorySize() {

		return (bean.getTotalPhysicalMemorySize()/(1024*1024));

	}

	//CPU for whole system
	public static double currentSystemCpuValue() {

		return Misc.round(bean.getSystemCpuLoad()*100, 1);

	}

	//CPU for the JVM
	public static String currentJvmCpuValue() {

		return ""+Misc.round(bean.getProcessCpuLoad()*100, 1);

	}
	
	@SuppressWarnings("unchecked")
	public static void addNewDay() {

		if (stat_date.size()==0 || !stat_date.get(stat_date.size()-1).equals(currentDate)) {
			
			if (stat_date.size()==0) {
				
				String cur = currentDate;
				for(int i=0;i<30;i++) {
					
					cur = Misc.dateDiffOneDay(cur);
					
					stat_date.add(0, cur);
					stat_tot_session.add(0, 0);
					stat_tot_command.add(0, 0);
					
				}
				
			}
			
			stat_date.add(currentDate);
			stat_tot_session.add(0);
			stat_tot_command.add(0);
		}

	}

	public static void deleteOldDay() {
		if (stat_date.size()>31) {
			stat_date.remove(0);
			stat_tot_session.remove(0); 
			stat_tot_command.remove(0); 
		}
	}

}