/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.tools;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.HashMap;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.Vector;
import java.util.regex.Pattern;

import org.json.simple.JSONArray;
import org.json.simple.JSONValue;

import re.jpayet.mentdb.core.basic.BasicResult;
import re.jpayet.mentdb.core.command.MQLValue;

//The miscellaneous class
public class Misc {
	
	public static String rpad_smallint(String str) {
		
		return str + ("     ".substring(str.length()));
		
	}
	
	public static String rpad_int(String str) {
		
		return str + ("          ".substring(str.length()));
		
	}
	
	public static String rpad_long(String str) {
		
		return str + ("                   ".substring(str.length()));
		
	}
	
	public static String deleteFile(String fileOrDirectoryPath) throws Exception {

		//Initialization
		String b = "1";
		File fileOrdir = new File(fileOrDirectoryPath);
		String[] dirList;
		String subFileOrDir = "";

		//Check if directory
		if (!fileOrdir.isDirectory()) {

			//It's a file, delete it
			fileOrdir.delete();

		} else {

			//It's a directory

			//Get all files or directories in the current directory
			dirList=fileOrdir.list();

			//Parse all files or directories
			for (int i=0; i<dirList.length; i++) {

				//Get the sub file or directory
				subFileOrDir = dirList[i];

				//Delete the sub file or directory
				if (b.equals("1")) {

					b = deleteFile(fileOrDirectoryPath+File.separator+subFileOrDir);

				}
			}

			//Delete the current directory
			fileOrdir.delete();
		}

		return b;

	}
	
	//Check if the command is valid or not
	public static boolean isCmdSession(String cmd0, Vector<MQLValue> param, org.eclipse.jetty.websocket.api.Session session, int size) throws IOException {

		if (param.size()<1) return false; 
		else if (param.get(0).value.toLowerCase().equals(cmd0)) {
			if (param.size()!=size) {
				BasicResult br = new BasicResult("", "ai", 0, "The number of parameter is not valid ("+size+", found "+param.size()+").", 0);
				session.getRemote().sendString(br.toString());
				return false;
			} else return true;
		} else return false;
		
	}
	
	//Get MD5 of a string
	public static String md5(String str) throws UnsupportedEncodingException, NoSuchAlgorithmException {
		
		byte[] bytesOfPassword = str.getBytes("UTF-8");

		MessageDigest md = MessageDigest.getInstance("MD5");
		return new String(md.digest(bytesOfPassword), "UTF-8");
		
	}
	
	//Get all objects in string tab format
	public static String[] JSONArray2StringTab(JSONArray in) throws Exception {
		
		//Initialization
		String[] result = new String[in.size()];
		
		//Parse all objects
		for(int i=0;i<in.size();i++) {
			
			//Get the current object
			result[i] = in.get(i)+"";
			
		}
		
		return result;
		
	}
	
	//Diff date
	public static String dateDiffOneDay(String date) {
		
		//Initialization
		Calendar cal = Calendar.getInstance();    
		cal.set(Calendar.DAY_OF_MONTH, Integer.parseInt(date.substring(8, 10)));
		cal.set(Calendar.MONTH, Integer.parseInt(date.substring(5, 7))-1);
		cal.set(Calendar.YEAR, Integer.parseInt(date.substring(0, 4)));
		cal.add(Calendar.DAY_OF_MONTH, -1);
		
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		
		return sdf.format(cal.getTime());
		
	}
	
	public static double round(double value, int places) {
	    if (places < 0) throw new IllegalArgumentException();

	    long factor = (long) Math.pow(10, places);
	    value = value * factor;
	    long tmp = Math.round(value);
	    return (double) tmp / factor;
	}
	
	//Load a JSON string
	public static JSONArray loadArray(String json) {
		
		return (JSONArray) JSONValue.parse(json);
		
	}
	
	public static int size(String atomList, String separator) {

		//Prepare list
		String copyAtomList=atomList;
		Pattern motif = Pattern.compile("["+separator+"]");

		//Split the list
		String[] ch = motif.split(copyAtomList, -1);

		//Return the size
		return ch.length;

	}
	
	//Vector message to string
	public static String vectorToStringMsg(Vector<MQLValue> inMsg) {
		
		//Initialization
		String result = "";
		
		//Parse all string
		for(int i=0;i<inMsg.size();i++) {
			
			//Concat all string
			if (inMsg.elementAt(i).value.indexOf(" ")>-1) result += " \""+inMsg.elementAt(i).value.replace("\"", "\\\"")+"\"";
			else result += " "+inMsg.elementAt(i).value;
			
		}
		
		//Delete the first char
		if (!result.equals("")) {
			
			result = result.substring(1);
			
		}
		
		return result;
		
	}
	
	//Split without empty
	public static Vector<Vector<MQLValue>> splitCommand(String str) throws Exception {
		
		//Initialization
		String tmpstr = str;
		Vector<Vector<MQLValue>> commands = new Vector<Vector<MQLValue>>();
		commands.add(new Vector<MQLValue>());
		int endIndex = 1;
		int depth = 0;
		String tmpCmd = "";

		//Parse the command string
		while (!tmpstr.equals("")) {

			//Check if the string begins by '
			if (tmpstr.substring(0, 1).equals(";")) {
				
				if (depth==0) {
					commands.get(commands.size()-1).add(new MQLValue(";", 0));
					commands.add(new Vector<MQLValue>());
				} else {
					tmpCmd+=";";
				}
				tmpstr = tmpstr.substring(1);
				
			} else if (tmpstr.substring(0, 1).equals("\n")) {
				
				if (depth==0) {
					
				} else {
					tmpCmd+="\n";
				}
				tmpstr = tmpstr.substring(1);
				
			} else if (tmpstr.substring(0, 1).equals(" ")) {
				
				if (depth==0) {
					
				} else {
					tmpCmd+=" ";
				}
				tmpstr = tmpstr.substring(1);
				
			} else if (tmpstr.substring(0, 1).equals("\t")) {
				
				if (depth==0) {
					
				} else {
					tmpCmd+="\t";
				}
				tmpstr = tmpstr.substring(1);
				
			} else if (tmpstr.substring(0, 1).equals("{")) {
				
				if (depth>0) {
					tmpCmd+="{";
				}
				depth += 1;
				tmpstr = tmpstr.substring(1);
				
			} else if (tmpstr.substring(0, 1).equals("(")) {
				
				if (depth>0) {
					tmpCmd+="(";
				}
				depth += 1;
				tmpstr = tmpstr.substring(1);
				
			} else if (tmpstr.substring(0, 1).equals("}")) {

				depth -= 1;
				
				if (depth==0) {
					
					if (!Misc.lrtrim(tmpCmd).endsWith(";")) {
						tmpCmd+=";";
					}

					if (commands.get(commands.size()-1).size()>0) {
						
						if (commands.get(commands.size()-1).get(0).value.equals("script")
								&& (commands.get(commands.size()-1).get(1).value.equals("add") 
										|| commands.get(commands.size()-1).get(1).value.equals("create")
										|| commands.get(commands.size()-1).get(1).value.equals("insert")
										|| commands.get(commands.size()-1).get(1).value.equals("update"))) {
							
							commands.get(commands.size()-1).add(new MQLValue(tmpCmd, 0));
							
						} else if (commands.get(commands.size()-1).get(0).value.equals("relation")
								&& commands.get(commands.size()-1).get(1).value.equals("set")
								&& (commands.get(commands.size()-1).get(2).value.equals("condition") 
										|| commands.get(commands.size()-1).get(2).value.equals("action"))) {
							
							commands.get(commands.size()-1).add(new MQLValue(tmpCmd, 0));
							
						} else if (commands.get(commands.size()-1).get(0).value.equals("relation")
								&& commands.get(commands.size()-1).get(1).value.equals("set")
								&& commands.get(commands.size()-1).get(2).value.equals("else")
								&& commands.get(commands.size()-1).get(3).value.equals("action")) {
							
							commands.get(commands.size()-1).add(new MQLValue(tmpCmd, 0));
							
						} else {
							
							switch (commands.get(commands.size()-1).get(0).value) {
							case "if": case "case": case "for": case "while": case "repeat": case "exception": case "try": case "and": case "or":
								commands.get(commands.size()-1).add(new MQLValue(tmpCmd, 0));
								break;
							default: 
								commands.get(commands.size()-1).add(new MQLValue(tmpCmd, 2));
							}
							
						}
						
					} else commands.get(commands.size()-1).add(new MQLValue(tmpCmd, 2));
					tmpCmd = "";
					
				} else {
					tmpCmd+="}";
				}
				tmpstr = tmpstr.substring(1);

			} else if (tmpstr.substring(0, 1).equals(")")) {

				depth -= 1;
				
				if (depth==0) {
					
					if (!Misc.lrtrim(tmpCmd).endsWith(";")) {
						tmpCmd+=";";
					}

					if (commands.get(commands.size()-1).size()>0) {
						
						if (commands.get(commands.size()-1).get(0).value.equals("script")
								&& (commands.get(commands.size()-1).get(1).value.equals("add") 
										|| commands.get(commands.size()-1).get(1).value.equals("create")
										|| commands.get(commands.size()-1).get(1).value.equals("insert")
										|| commands.get(commands.size()-1).get(1).value.equals("update"))) {
							
							commands.get(commands.size()-1).add(new MQLValue(tmpCmd, 0));
							
						} else if (commands.get(commands.size()-1).get(0).value.equals("relation")
								&& commands.get(commands.size()-1).get(1).value.equals("set")
								&& (commands.get(commands.size()-1).get(2).value.equals("condition") 
										|| commands.get(commands.size()-1).get(2).value.equals("action"))) {
							
							commands.get(commands.size()-1).add(new MQLValue(tmpCmd, 0));
							
						} else if (commands.get(commands.size()-1).get(0).value.equals("relation")
								&& commands.get(commands.size()-1).get(1).value.equals("set")
								&& commands.get(commands.size()-1).get(2).value.equals("else")
								&& commands.get(commands.size()-1).get(3).value.equals("action")) {
							
							commands.get(commands.size()-1).add(new MQLValue(tmpCmd, 0));
							
						} else {
							
							switch (commands.get(commands.size()-1).get(0).value) {
							case "if": case "case": case "for": case "while": case "repeat": case "exception": case "try": case "and": case "or":
								commands.get(commands.size()-1).add(new MQLValue(tmpCmd, 0));
								break;
							default: 
								commands.get(commands.size()-1).add(new MQLValue(tmpCmd, 2));
							}
							
						}
						
					} else commands.get(commands.size()-1).add(new MQLValue(tmpCmd, 2));
					tmpCmd = "";
					
				} else {
					tmpCmd+=")";
				}
				tmpstr = tmpstr.substring(1);

			} else if (tmpstr.substring(0, 1).equals("\"")) {

				//It is a string
				endIndex = tmpstr.indexOf("\"", 1);
				while (endIndex!=-1 && tmpstr.substring(endIndex-1, endIndex).equals("\\")) {
					endIndex = tmpstr.indexOf("\"", endIndex+1);
				}
				
				//Generate an error if the string is never close
				if (endIndex==-1) {
					
					throw new Exception("Sorry, Invalid command (string must be close)");
					
				} else {
					
					//The string is close
					String tmptmpstr = tmpstr.substring(1, endIndex);
					if (depth>0) {
						tmpCmd += "\""+tmptmpstr+"\"";
					} else {
						commands.get(commands.size()-1).add(new MQLValue(tmptmpstr.replace("\\\"", "\""), 0));
					}
					tmpstr = tmpstr.substring(endIndex+1);
					
				}
				
			} else {

				//It is not a string
				//Find the space
				HashMap<Integer, Integer> hm = new HashMap<Integer, Integer>();
				int i0 = tmpstr.indexOf("\t");hm.put(i0, 0);
				int i1 = tmpstr.indexOf(" ");hm.put(i1, 0);
				int i2 = tmpstr.indexOf(";");hm.put(i2, 0);
				int i3 = tmpstr.indexOf("{");hm.put(i3, 0);
				int i4 = tmpstr.indexOf("}");hm.put(i4, 0);
				int i5 = tmpstr.indexOf("(");hm.put(i5, 0);
				int i6 = tmpstr.indexOf(")");hm.put(i6, 0);
				int i7 = tmpstr.indexOf("\n");hm.put(i7, 0);
				
				if (hm.size()==1) {
					endIndex = -1;
				} else {
					SortedSet<Integer> keys = new TreeSet<Integer>(hm.keySet());
					Object[] iKeys = keys.toArray();
					if (((Integer) iKeys[0])!=-1) {
						endIndex = (Integer) iKeys[0];
					} else {
						endIndex = (Integer) iKeys[1];
					}
					
				}
								
				//Go to the next key
				if (endIndex==-1) {
					String tmptmpstr = tmpstr.substring(0, tmpstr.length());
					if (!tmptmpstr.equals("")) {

						if (tmptmpstr.startsWith("@[") && tmptmpstr.endsWith("]")) {
							if (depth>0) {
								tmpCmd += tmptmpstr;
							} else {
								commands.get(commands.size()-1).add(new MQLValue(tmptmpstr, -1));
							}
						} else if (tmptmpstr.startsWith("[") && tmptmpstr.endsWith("]")) {
							if (depth>0) {
								tmpCmd += tmptmpstr;
							} else {
								commands.get(commands.size()-1).add(new MQLValue(tmptmpstr, 1));
							}
						} else {
							if (depth>0) {
								tmpCmd += tmptmpstr;
							} else {
								commands.get(commands.size()-1).add(new MQLValue(tmptmpstr, 0));
							}
						}
						
					}
					tmpstr = "";
					
				} else {

					String tmptmpstr = tmpstr.substring(0, endIndex);
					if (!tmptmpstr.equals("")) {
						if (tmptmpstr.startsWith("@[") && tmptmpstr.endsWith("]")) {
							if (depth>0) {
								tmpCmd += tmptmpstr;
							} else {
								commands.get(commands.size()-1).add(new MQLValue(tmptmpstr, -1));
							}
						} else if (tmptmpstr.startsWith("[") && tmptmpstr.endsWith("]")) {
							if (depth>0) {
								tmpCmd += tmptmpstr;
							} else {
								commands.get(commands.size()-1).add(new MQLValue(tmptmpstr, 1));
							}
						} else {
							if (depth>0) {
								tmpCmd += tmptmpstr;
							} else {
								commands.get(commands.size()-1).add(new MQLValue(tmptmpstr, 0));
							}
						}
					}
					
					tmpstr = tmpstr.substring(endIndex);
					
				}
				
			}
			
		}

		if (depth!=0) {
			throw new Exception("Sorry, Invalid command ('{' or '}' or '(' or ')' must be close)");
		}
		
		//Remove the last if empty
		int nbDeleted = 0, nbTab = commands.size();
		for(int i=0;i<nbTab;i++) {
			if (commands.get(i-nbDeleted).size()==0) {
				commands.remove(i-nbDeleted);
				nbDeleted++;
			}
		}
		
		//Return all command lines
		return commands;
		
	}
	
	//Split words
	public static String[] splitWords(String words, String separator) {
		
		return words.split(Pattern.quote(separator));
		
	}
	
	//Read an ini file
	public static String conf_value(String path, String section, String field) {

		//Initialization
		String line="", value="";
		int found = 0;
		BufferedReader in =null;

		//Try to get the value
		try {

			//In stream
			in =  new BufferedReader( new FileReader(path));

			//parse to find the section
			while ((line=in.readLine())!=null)
			{
				
				if (!line.equals("") && line.equals("[" + section + "]"))
				{
					//The section found
					break;
				}
				
			}

			//parse to find the field
			while (((line=in.readLine())!=null))
			{
				if (!line.equals("") && !(line.substring(0,1).equals("["))) {
					if (atom(line, 1, "=").equals(field))
					{
						//The field found
						found=1;
						break;
					}
					
				}
			}
			
			//Manage the last line
			if (found==0 && line!=null &&  !line.equals("")) {
				if (atom(line, 1, "=").equals(field))	{
					found=1;
				}
			}
			
			//Get the value of found
			if (found==1) value=line.substring(field.length()+1).replace("\r\n", "").replace("\n", "").replace("\r", "");

			//return the value
			return value;

		} catch (Exception e) {

			return null;

		} finally {

			//Close the stream
			try {in.close();} catch (Exception e) {};

		}

	}
	
	//Basic rpad function
	public static String rpad(String str, String padString, String paddedLength) {

		//Initialization
		String strTmp=str;

		//Try to complete the string
		try {

			//Return null if one parameter is null
			if (str==null || padString==null || paddedLength==null) {

				return null;

			} else {

				//Complete the string
				for(int i=str.length();i<Integer.parseInt(paddedLength);i++) {
					strTmp+=padString;
				}

				//Return the right pad string
				return strTmp;

			}

		} catch (Exception e) {

			//If an error was generated then return null
			return null;

		}

	}
	
	static String current_system_out_print = "";
	
	//Add space rpad
	public static void system_out_print(String str, boolean ln, String color, String strAdd) {
		
		str += strAdd;
		
		if (ln) {
			
			System.out.println(color+Misc.rpad(current_system_out_print+str, " ", "65").substring((current_system_out_print+str).length())+str.substring(0, str.length()-strAdd.length()));
			
		} else {
			current_system_out_print = str;
			System.out.print(color+str);
		}

	}
	
	//Get the os
	public static String os() {
		
		String OS = System.getProperty("os.name").toLowerCase();
		
		if (OS.indexOf("win") >= 0) {
			
			return "win";
			
		} else if (OS.indexOf("mac") >= 0) {
			
			return "mac";
			
		} else if (OS.indexOf("nix") >= 0 || OS.indexOf("nux") >= 0 || OS.indexOf("aix") > 0 ) {
			
			return "unix";
			
		} else return "unknow";
		
	}
	
	//Get a specific string in a list
	public static String atom(String atomList, int index, String separator) {
		
		//Prepare the list
		String copyAtomList=atomList;
		Pattern motif = Pattern.compile("["+separator+"]");
		
		//Split the list
		String[] ch = motif.split(copyAtomList, -1);
		
		//Return the atom
		return ch[index-1];
		
	}
	
	//Get a specific string in a list
	public static Pattern motif = Pattern.compile("[;]");
	public static String[] atom_tab(String atomList) {
		
		//Prepare the list
		String copyAtomList=atomList;
		
		//Return the list
		return motif.split(copyAtomList, -1);
		
	}
	
	//Basic lrtrim function
	public static String lrtrim(String str) {
		
		return str.replaceAll("\\s+$", "").replaceAll("^\\s+", "");

	}
	
	//Basic trim function
	public static String trim(String str) {
		
		return str.replaceAll("\\s+$", "").replaceAll("^\\s+", "").replaceAll("\\b\\s{2,}\\b", " ");

	}
	
	public static String load(String filePath) throws Exception
	{

		//Initialization
		String line="";
		BufferedReader in = null;
		StringBuilder data = new StringBuilder();

		//Try to get data
		try {

			//Load the stream
			in =  new BufferedReader( new InputStreamReader( new FileInputStream(filePath)));

			//Parse the stream
			while ((line=in.readLine())!=null)
			{
				//Get line by line
				data.append(line);
				data.append("\n");
			}

		} catch (Exception e) {

			//Generate an error
			throw e;

		} finally {

			//Close input stream
			try {in.close();} catch (Exception e) {};

		}

		//Return the data
		return data.toString();
	}
	
	public static String create(String filePath, String str) throws Exception {

		//Initialization
		FileWriter fw = null;

		//Initialization
		File file = new File(filePath);

		try {

			//Create the new file
			file.createNewFile();
			fw = new FileWriter(file);

			//Write str
			fw.write(str);

			//Flush the file
			fw.flush();

			//Return true
			return "1";

		} catch (Exception e) {

			//Generate an error
			throw e;

		} finally {

			//Close the writer
			try {fw.close();} catch (Exception e) {}

		}

	}

}